"""SEOService - automatic SEO metadata generation."""

import json
import re
from typing import Any

from ..models import Entity
from .entity import EntityService
from .field import field_service


class SEOService:
    """
    SEO metadata generator.

    Automatically generates:
    - title, description
    - OGP meta tags
    - JSON-LD structured data
    - sitemap.xml
    """

    def __init__(self, entity_svc: EntityService, site_url: str = "", site_settings: dict = None):
        self.entity_svc = entity_svc
        self.site_url = site_url.rstrip("/")
        self.site_settings = site_settings or {}

    def generate_meta(self, entity: Entity) -> dict[str, Any]:
        """Generate SEO meta tags for an entity."""
        data = self.entity_svc.serialize(entity)
        ct = field_service.get_content_type(entity.type)

        # Use page-specific SEO settings if available
        title = data.get("seo_title") or self._get_title(data, ct)
        description = data.get("seo_description") or self._get_description(data, ct)
        canonical_url = data.get("seo_canonical") or self._get_url(entity, data)
        image = self._get_image(data)

        # OGP specific overrides
        og_title = data.get("og_title") or title
        og_description = data.get("og_description") or description
        og_image = data.get("og_image") or image

        # noindex/nofollow
        robots = []
        if data.get("seo_noindex"):
            robots.append("noindex")
        if data.get("seo_nofollow"):
            robots.append("nofollow")

        return {
            "title": title,
            "description": description,
            "canonical": canonical_url,
            "robots": ", ".join(robots) if robots else None,
            "ogp": self._generate_ogp(
                og_title, og_description, canonical_url, og_image, entity.type, data
            ),
            "json_ld": self._generate_json_ld(
                entity, data, title, description, canonical_url, image
            ),
        }

    def _get_title(self, data: dict, ct) -> str:
        """Extract title from entity data."""
        for field in ["title", "name"]:
            if field in data and data[field]:
                return str(data[field])
        return ct.label if ct else "Untitled"

    def _get_description(self, data: dict, ct) -> str:
        """Extract or generate description."""
        # Try explicit fields
        for field in ["description", "excerpt", "summary"]:
            if field in data and data[field]:
                return self._truncate(str(data[field]), 160)

        # Try to extract from body/content
        for field in ["body", "content"]:
            if field in data and data[field]:
                text = self._extract_text(data[field])
                if text:
                    return self._truncate(text, 160)

        return ""

    def _get_url(self, entity: Entity, data: dict) -> str:
        """Generate canonical URL."""
        slug = data.get("slug", entity.id)
        return f"{self.site_url}/{entity.type}/{slug}"

    def _get_image(self, data: dict) -> str | None:
        """Extract featured image."""
        for field in ["featured_image", "image", "thumbnail", "og_image"]:
            if field in data and data[field]:
                img = data[field]
                if isinstance(img, str):
                    if img.startswith("/"):
                        return f"{self.site_url}{img}"
                    return img
        return None

    def _generate_ogp(
        self,
        title: str,
        description: str,
        url: str,
        image: str | None,
        entity_type: str,
        entity_data: dict = None,
    ) -> dict[str, str]:
        """Generate Open Graph Protocol meta tags."""
        og_type = "article" if entity_type in ("post", "page") else "website"
        data = entity_data or {}

        ogp = {
            "og:type": og_type,
            "og:title": title,
            "og:description": description,
            "og:url": url,
        }

        # Add site name and locale from settings
        if self.site_settings.get("name"):
            ogp["og:site_name"] = self.site_settings["name"]
        ogp["og:locale"] = self.site_settings.get("locale", "ja_JP")

        if image:
            ogp["og:image"] = image

        # Article-specific tags
        if og_type == "article":
            if data.get("created_at"):
                ogp["article:published_time"] = data["created_at"]
            if data.get("updated_at"):
                ogp["article:modified_time"] = data["updated_at"]
            # Author info if available
            if data.get("author_name"):
                ogp["article:author"] = data["author_name"]

        # Twitter Card
        ogp["twitter:card"] = "summary_large_image" if image else "summary"
        ogp["twitter:title"] = title
        ogp["twitter:description"] = description
        if image:
            ogp["twitter:image"] = image
        # Twitter site from settings
        if self.site_settings.get("twitter_site"):
            ogp["twitter:site"] = self.site_settings["twitter_site"]
        # Twitter creator (author's twitter handle if available)
        if data.get("twitter_creator"):
            ogp["twitter:creator"] = data["twitter_creator"]

        return ogp

    def _generate_json_ld(
        self,
        entity: Entity,
        data: dict,
        title: str,
        description: str,
        url: str,
        image: str | None,
    ) -> dict[str, Any]:
        """Generate JSON-LD structured data."""
        if entity.type == "post":
            return self._json_ld_article(entity, data, title, description, url, image)
        elif entity.type == "page":
            return self._json_ld_webpage(title, description, url)
        else:
            return self._json_ld_thing(title, description, url)

    def _json_ld_article(
        self,
        entity: Entity,
        data: dict,
        title: str,
        description: str,
        url: str,
        image: str | None,
    ) -> dict[str, Any]:
        """Generate Article JSON-LD."""
        ld = {
            "@context": "https://schema.org",
            "@type": "Article",
            "headline": title,
            "description": description,
            "url": url,
            "datePublished": entity.created_at.isoformat() if entity.created_at else None,
            "dateModified": entity.updated_at.isoformat() if entity.updated_at else None,
        }

        if image:
            ld["image"] = image

        # Author info would come from relations
        # For now, use a placeholder
        ld["author"] = {
            "@type": "Person",
            "name": "Author",
        }

        return ld

    def _json_ld_webpage(
        self,
        title: str,
        description: str,
        url: str,
    ) -> dict[str, Any]:
        """Generate WebPage JSON-LD."""
        return {
            "@context": "https://schema.org",
            "@type": "WebPage",
            "name": title,
            "description": description,
            "url": url,
        }

    def _json_ld_thing(
        self,
        title: str,
        description: str,
        url: str,
    ) -> dict[str, Any]:
        """Generate generic Thing JSON-LD."""
        return {
            "@context": "https://schema.org",
            "@type": "Thing",
            "name": title,
            "description": description,
            "url": url,
        }

    def _extract_text(self, content: Any) -> str:
        """Extract plain text from content (Editor.js JSON or HTML)."""
        if not content:
            return ""

        data = content

        # Parse JSON string if needed
        if isinstance(content, str):
            try:
                data = json.loads(content)
            except (json.JSONDecodeError, TypeError):
                # Treat as HTML/plain text
                return self._strip_html(content)

        # Handle Editor.js blocks (dict with "blocks" key)
        if isinstance(data, dict) and "blocks" in data:
            texts = []
            for block in data["blocks"]:
                if block.get("type") == "paragraph":
                    text = block.get("data", {}).get("text", "")
                    texts.append(self._strip_html(text))
                elif block.get("type") == "header":
                    text = block.get("data", {}).get("text", "")
                    texts.append(self._strip_html(text))
            return " ".join(texts)

        # Fallback: convert to string
        if isinstance(data, str):
            return self._strip_html(data)

        return str(content)

    def _strip_html(self, text: str) -> str:
        """Remove HTML tags from text."""
        return re.sub(r"<[^>]+>", "", text)

    def _truncate(self, text: str, max_length: int) -> str:
        """Truncate text to max length."""
        text = text.strip()
        if len(text) <= max_length:
            return text
        return text[: max_length - 3].rsplit(" ", 1)[0] + "..."

    async def generate_sitemap(self, content_types: list[str] = None) -> str:
        """Generate sitemap.xml content."""
        if content_types is None:
            content_types = ["post", "page"]

        urls = []

        for ct_name in content_types:
            entities = await self.entity_svc.find(
                ct_name,
                limit=10000,
                filters={"status": "published"} if ct_name == "post" else {},
            )

            for entity in entities:
                data = self.entity_svc.serialize(entity)
                url = self._get_url(entity, data)
                lastmod = entity.updated_at.strftime("%Y-%m-%d") if entity.updated_at else ""

                urls.append(
                    {
                        "loc": url,
                        "lastmod": lastmod,
                        "changefreq": "weekly" if ct_name == "post" else "monthly",
                        "priority": "0.8" if ct_name == "post" else "0.5",
                    }
                )

        # Generate XML
        xml_parts = ['<?xml version="1.0" encoding="UTF-8"?>']
        xml_parts.append('<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">')

        for url_data in urls:
            xml_parts.append("  <url>")
            xml_parts.append(f"    <loc>{url_data['loc']}</loc>")
            if url_data["lastmod"]:
                xml_parts.append(f"    <lastmod>{url_data['lastmod']}</lastmod>")
            xml_parts.append(f"    <changefreq>{url_data['changefreq']}</changefreq>")
            xml_parts.append(f"    <priority>{url_data['priority']}</priority>")
            xml_parts.append("  </url>")

        xml_parts.append("</urlset>")

        return "\n".join(xml_parts)

    def generate_site_json_ld(self) -> list[dict[str, Any]]:
        """Generate site-wide JSON-LD (Organization, WebSite)."""
        schemas = []

        site_name = self.site_settings.get("name", "Focomy")
        site_description = self.site_settings.get("description", "")

        # Organization schema
        org = {
            "@context": "https://schema.org",
            "@type": "Organization",
            "name": site_name,
            "url": self.site_url,
        }
        if self.site_settings.get("logo"):
            org["logo"] = self.site_settings["logo"]
        if self.site_settings.get("email"):
            org["email"] = self.site_settings["email"]
        if self.site_settings.get("social_links"):
            org["sameAs"] = self.site_settings["social_links"]
        schemas.append(org)

        # WebSite schema with SearchAction
        website = {
            "@context": "https://schema.org",
            "@type": "WebSite",
            "name": site_name,
            "url": self.site_url,
            "potentialAction": {
                "@type": "SearchAction",
                "target": {
                    "@type": "EntryPoint",
                    "urlTemplate": f"{self.site_url}/search?q={{search_term_string}}",
                },
                "query-input": "required name=search_term_string",
            },
        }
        if site_description:
            website["description"] = site_description
        schemas.append(website)

        return schemas

    def generate_person_json_ld(self, author_data: dict) -> dict[str, Any]:
        """Generate Person JSON-LD for author."""
        person = {
            "@context": "https://schema.org",
            "@type": "Person",
            "name": author_data.get("name", "Author"),
        }
        if author_data.get("email"):
            person["email"] = author_data["email"]
        if author_data.get("url"):
            person["url"] = author_data["url"]
        if author_data.get("image"):
            person["image"] = author_data["image"]
        if author_data.get("description"):
            person["description"] = author_data["description"]
        if author_data.get("social_links"):
            person["sameAs"] = author_data["social_links"]
        return person

    def generate_faq_json_ld(self, faq_items: list[dict]) -> dict[str, Any]:
        """Generate FAQPage JSON-LD.

        Args:
            faq_items: List of {"question": str, "answer": str}
        """
        return {
            "@context": "https://schema.org",
            "@type": "FAQPage",
            "mainEntity": [
                {
                    "@type": "Question",
                    "name": item["question"],
                    "acceptedAnswer": {"@type": "Answer", "text": item["answer"]},
                }
                for item in faq_items
            ],
        }

    def generate_breadcrumb_json_ld(self, items: list[dict]) -> dict[str, Any]:
        """Generate BreadcrumbList JSON-LD.

        Args:
            items: List of {"name": str, "url": str}
        """
        return {
            "@context": "https://schema.org",
            "@type": "BreadcrumbList",
            "itemListElement": [
                {"@type": "ListItem", "position": i + 1, "name": item["name"], "item": item["url"]}
                for i, item in enumerate(items)
            ],
        }

    def render_meta_tags(self, meta: dict) -> str:
        """Render meta tags as HTML."""
        lines = []

        # Basic meta
        if meta.get("title"):
            lines.append(f'<title>{meta["title"]}</title>')
        if meta.get("description"):
            lines.append(f'<meta name="description" content="{meta["description"]}">')
        if meta.get("canonical"):
            lines.append(f'<link rel="canonical" href="{meta["canonical"]}">')
        if meta.get("robots"):
            lines.append(f'<meta name="robots" content="{meta["robots"]}">')

        # OGP
        for key, value in meta.get("ogp", {}).items():
            if key.startswith("twitter:"):
                lines.append(f'<meta name="{key}" content="{value}">')
            else:
                lines.append(f'<meta property="{key}" content="{value}">')

        # JSON-LD
        if meta.get("json_ld"):
            json_ld = json.dumps(meta["json_ld"], ensure_ascii=False)
            lines.append(f'<script type="application/ld+json">{json_ld}</script>')

        return "\n".join(lines)

    def render_site_json_ld(self) -> str:
        """Render site-wide JSON-LD as HTML script tags."""
        schemas = self.generate_site_json_ld()
        lines = []
        for schema in schemas:
            json_ld = json.dumps(schema, ensure_ascii=False)
            lines.append(f'<script type="application/ld+json">{json_ld}</script>')
        return "\n".join(lines)



"""LinkValidatorService - detect broken links and orphan pages."""

import asyncio
import json
import re

import httpx
from sqlalchemy.ext.asyncio import AsyncSession

from .entity import EntityService
from .field import field_service


class LinkValidatorService:
    """
    Link validation service.

    Detects:
    - Broken internal links (404s)
    - Broken external links (connection errors, 4xx/5xx)
    - Orphan pages (pages not linked from anywhere)
    """

    def __init__(self, db: AsyncSession, site_url: str = ""):
        self.db = db
        self.entity_svc = EntityService(db)
        self.site_url = site_url.rstrip("/")

    async def validate_all_links(self, check_external: bool = False) -> dict:
        """
        Validate all links across published content.

        Args:
            check_external: Whether to check external URLs (slow)

        Returns:
            Dict with broken_links, orphan_pages, and stats
        """
        # Collect all pages and their outbound links
        pages = await self._get_all_published_pages()
        all_links = await self._extract_all_links(pages)
        internal_pages = {p["url"] for p in pages}

        broken_links = []
        external_errors = []

        # Check internal links
        for link in all_links["internal"]:
            target_path = link["href"]
            # Normalize path
            if not target_path.startswith("/"):
                continue

            # Check if the target exists
            if not await self._internal_link_exists(target_path, internal_pages):
                broken_links.append(
                    {
                        "source_url": link["source_url"],
                        "source_title": link["source_title"],
                        "source_id": link["source_id"],
                        "target_url": target_path,
                        "type": "internal",
                        "status": "not_found",
                    }
                )

        # Check external links if requested
        if check_external:
            external_results = await self._check_external_links(all_links["external"])
            for result in external_results:
                if result["status"] != "ok":
                    external_errors.append(result)

        return {
            "broken_links": broken_links,
            "external_errors": external_errors if check_external else [],
            "stats": {
                "total_pages": len(pages),
                "total_internal_links": len(all_links["internal"]),
                "total_external_links": len(all_links["external"]),
                "broken_internal": len(broken_links),
                "broken_external": len(external_errors) if check_external else 0,
            },
        }

    async def find_orphan_pages(self) -> list[dict]:
        """
        Find pages that are not linked from anywhere.

        Returns:
            List of orphan pages with their details
        """
        pages = await self._get_all_published_pages()
        all_links = await self._extract_all_links(pages)

        # Build set of linked pages
        linked_urls = set()
        for link in all_links["internal"]:
            href = link["href"]
            if href.startswith("/"):
                linked_urls.add(href)

        # Find pages not in the linked set
        orphans = []
        for page in pages:
            page_url = page["url"]
            # Skip home page
            if page_url == "/":
                continue

            if page_url not in linked_urls:
                orphans.append(
                    {
                        "id": page["id"],
                        "title": page["title"],
                        "url": page_url,
                        "type": page["type"],
                        "created_at": page.get("created_at", ""),
                    }
                )

        return orphans

    async def validate_single_page(self, entity_id: str) -> dict:
        """Validate links in a single page."""
        entity = await self.entity_svc.get(entity_id)
        if not entity:
            return {"error": "Entity not found"}

        data = self.entity_svc.serialize(entity)
        links = self._extract_links_from_entity(entity.id, entity.type, data)

        broken = []
        pages = await self._get_all_published_pages()
        internal_pages = {p["url"] for p in pages}

        for link in links["internal"]:
            if not await self._internal_link_exists(link["href"], internal_pages):
                broken.append(
                    {
                        "target_url": link["href"],
                        "type": "internal",
                        "status": "not_found",
                    }
                )

        return {
            "entity_id": entity_id,
            "broken_links": broken,
            "total_links": len(links["internal"]) + len(links["external"]),
        }

    async def _get_all_published_pages(self) -> list[dict]:
        """Get all published pages and posts."""
        pages = []

        for ct_name in ["post", "page"]:
            ct = field_service.get_content_type(ct_name)
            if not ct:
                continue

            entities = await self.entity_svc.find(
                ct_name,
                limit=10000,
                filters={"status": "published"},
            )

            for entity in entities:
                data = self.entity_svc.serialize(entity)
                slug = data.get("slug", entity.id)
                path_prefix = ct.path_prefix.strip("/") if ct.path_prefix else ct_name

                pages.append(
                    {
                        "id": entity.id,
                        "type": ct_name,
                        "title": data.get("title", data.get("name", "Untitled")),
                        "url": f"/{path_prefix}/{slug}",
                        "data": data,
                        "created_at": data.get("created_at", ""),
                    }
                )

        # Also add category pages
        categories = await self.entity_svc.find("category", limit=1000)
        for cat in categories:
            data = self.entity_svc.serialize(cat)
            slug = data.get("slug", cat.id)
            pages.append(
                {
                    "id": cat.id,
                    "type": "category",
                    "title": data.get("name", "Untitled"),
                    "url": f"/category/{slug}",
                    "data": data,
                    "created_at": data.get("created_at", ""),
                }
            )

        return pages

    async def _extract_all_links(self, pages: list[dict]) -> dict:
        """Extract all links from all pages."""
        internal_links = []
        external_links = []

        for page in pages:
            links = self._extract_links_from_entity(
                page["id"],
                page["type"],
                page["data"],
            )

            for link in links["internal"]:
                link["source_url"] = page["url"]
                link["source_title"] = page["title"]
                link["source_id"] = page["id"]
                internal_links.append(link)

            for link in links["external"]:
                link["source_url"] = page["url"]
                link["source_title"] = page["title"]
                link["source_id"] = page["id"]
                external_links.append(link)

        return {
            "internal": internal_links,
            "external": external_links,
        }

    def _extract_links_from_entity(
        self,
        entity_id: str,
        entity_type: str,
        data: dict,
    ) -> dict:
        """Extract links from entity content."""
        internal = []
        external = []

        # Check body/content field (Editor.js format or HTML)
        for field in ["body", "content"]:
            if field not in data:
                continue

            content = data[field]
            if not content:
                continue

            # Parse Editor.js JSON
            if isinstance(content, str):
                try:
                    content = json.loads(content)
                except (json.JSONDecodeError, TypeError):
                    # Treat as HTML
                    links = self._extract_links_from_html(content)
                    for href in links:
                        if self._is_internal_link(href):
                            internal.append({"href": self._normalize_path(href)})
                        else:
                            external.append({"href": href})
                    continue

            # Process Editor.js blocks
            if isinstance(content, dict) and "blocks" in content:
                for block in content.get("blocks", []):
                    block_links = self._extract_links_from_block(block)
                    for href in block_links:
                        if self._is_internal_link(href):
                            internal.append({"href": self._normalize_path(href)})
                        else:
                            external.append({"href": href})

        return {"internal": internal, "external": external}

    def _extract_links_from_block(self, block: dict) -> list[str]:
        """Extract links from an Editor.js block."""
        links = []
        block_type = block.get("type", "")
        block_data = block.get("data", {})

        if block_type == "paragraph":
            text = block_data.get("text", "")
            links.extend(self._extract_links_from_html(text))
        elif block_type == "header":
            text = block_data.get("text", "")
            links.extend(self._extract_links_from_html(text))
        elif block_type == "list":
            for item in block_data.get("items", []):
                if isinstance(item, str):
                    links.extend(self._extract_links_from_html(item))
                elif isinstance(item, dict):
                    links.extend(self._extract_links_from_html(item.get("content", "")))
        elif block_type == "linkTool":
            link = block_data.get("link", "")
            if link:
                links.append(link)
        elif block_type == "image":
            # Image URLs are typically internal/CDN, skip for link checking
            pass

        return links

    def _extract_links_from_html(self, html: str) -> list[str]:
        """Extract href values from HTML content."""
        if not html:
            return []

        # Simple regex to extract href attributes
        pattern = r'href=["\']([^"\']+)["\']'
        matches = re.findall(pattern, html, re.IGNORECASE)
        return matches

    def _is_internal_link(self, href: str) -> bool:
        """Check if a link is internal."""
        if not href:
            return False

        # Relative paths are internal
        if href.startswith("/"):
            return True

        # Check if it's the same domain
        if self.site_url and href.startswith(self.site_url):
            return True

        # External
        if href.startswith(("http://", "https://", "//")):
            return False

        # Anchors, mailto, tel, etc.
        if href.startswith(("#", "mailto:", "tel:", "javascript:")):
            return False

        # Assume relative paths are internal
        return True

    def _normalize_path(self, href: str) -> str:
        """Normalize a path for comparison."""
        # Remove site URL prefix
        if self.site_url and href.startswith(self.site_url):
            href = href[len(self.site_url) :]

        # Ensure starts with /
        if not href.startswith("/"):
            href = "/" + href

        # Remove trailing slash (except for root)
        if href != "/" and href.endswith("/"):
            href = href[:-1]

        # Remove query string and fragment
        if "?" in href:
            href = href.split("?")[0]
        if "#" in href:
            href = href.split("#")[0]

        return href

    async def _internal_link_exists(
        self,
        path: str,
        known_pages: set[str],
    ) -> bool:
        """Check if an internal link target exists."""
        # Normalize path
        path = self._normalize_path(path)

        # Check in known pages
        if path in known_pages:
            return True

        # Check special routes
        special_routes = [
            "/",
            "/search",
            "/feed.xml",
            "/atom.xml",
            "/feed.json",
            "/sitemap.xml",
            "/robots.txt",
            "/manifest.json",
        ]

        if path in special_routes:
            return True

        # Check archive routes
        if re.match(r"^/archive/\d{4}/\d{1,2}$", path):
            return True

        return False

    async def _check_external_links(self, links: list[dict]) -> list[dict]:
        """Check external links for accessibility."""
        results = []

        # Deduplicate URLs
        unique_urls = {}
        for link in links:
            href = link["href"]
            if href not in unique_urls:
                unique_urls[href] = link

        async with httpx.AsyncClient(timeout=10.0, follow_redirects=True) as client:
            for href, link in unique_urls.items():
                try:
                    response = await client.head(href)
                    if response.status_code >= 400:
                        results.append(
                            {
                                "source_url": link["source_url"],
                                "source_title": link["source_title"],
                                "source_id": link["source_id"],
                                "target_url": href,
                                "type": "external",
                                "status": f"http_{response.status_code}",
                            }
                        )
                    else:
                        results.append(
                            {
                                "target_url": href,
                                "type": "external",
                                "status": "ok",
                            }
                        )
                except httpx.TimeoutException:
                    results.append(
                        {
                            "source_url": link["source_url"],
                            "source_title": link["source_title"],
                            "source_id": link["source_id"],
                            "target_url": href,
                            "type": "external",
                            "status": "timeout",
                        }
                    )
                except httpx.RequestError:
                    results.append(
                        {
                            "source_url": link["source_url"],
                            "source_title": link["source_title"],
                            "source_id": link["source_id"],
                            "target_url": href,
                            "type": "external",
                            "status": "connection_error",
                        }
                    )

                # Rate limit
                await asyncio.sleep(0.1)

        return results



"""Preview Service - Import a few items for verification before full import."""

from __future__ import annotations

import logging
from dataclasses import dataclass, field
from datetime import datetime, timezone

from ...utils import utcnow
from typing import Any

from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from ...models import Entity, EntityValue, ImportJob
from ..entity import EntityService
from .constants import WP_STATUS_MAP
from .content_sanitizer import ContentSanitizer
from .wxr_parser import WXRData

logger = logging.getLogger(__name__)


@dataclass
class PreviewItem:
    """Imported preview item."""

    entity_type: str
    entity_id: str
    wp_id: int | str
    title: str
    slug: str
    status: str  # success, error
    message: str = ""


@dataclass
class PreviewResult:
    """Result of preview import."""

    job_id: str
    preview_id: str  # Unique ID for this preview session
    generated_at: datetime = field(default_factory=datetime.utcnow)
    items: list[PreviewItem] = field(default_factory=list)
    success_count: int = 0
    error_count: int = 0
    errors: list[str] = field(default_factory=list)
    committed: bool = False  # Whether preview was committed
    rolled_back: bool = False  # Whether preview was rolled back

    @property
    def has_errors(self) -> bool:
        """Check if there are any errors."""
        return self.error_count > 0

    def to_dict(self) -> dict:
        """Convert to dictionary for API response."""
        return {
            "job_id": self.job_id,
            "preview_id": self.preview_id,
            "generated_at": self.generated_at.isoformat(),
            "items": [
                {
                    "entity_type": i.entity_type,
                    "entity_id": i.entity_id,
                    "wp_id": i.wp_id,
                    "title": i.title,
                    "slug": i.slug,
                    "status": i.status,
                    "message": i.message,
                }
                for i in self.items
            ],
            "success_count": self.success_count,
            "error_count": self.error_count,
            "errors": self.errors,
            "has_errors": self.has_errors,
            "committed": self.committed,
            "rolled_back": self.rolled_back,
        }


class PreviewService:
    """
    Preview import service.

    Imports a small number of items to verify before full import.
    Preview items can be committed or rolled back.
    """

    DEFAULT_PREVIEW_COUNT = 3

    def __init__(self, db: AsyncSession):
        self.db = db
        self.entity_svc = EntityService(db)
        self.sanitizer = ContentSanitizer()
        self._preview_entities: dict[str, list[str]] = {}  # preview_id -> entity_ids

    async def run_preview(
        self,
        job_id: str,
        wxr_data: WXRData,
        count: int = DEFAULT_PREVIEW_COUNT,
    ) -> PreviewResult:
        """
        Run preview import.

        Args:
            job_id: Import job ID
            wxr_data: Parsed WordPress data
            count: Number of items to preview (default 3)

        Returns:
            PreviewResult with imported items
        """
        import uuid

        preview_id = str(uuid.uuid4())[:8]
        result = PreviewResult(job_id=job_id, preview_id=preview_id)
        imported_entity_ids: list[str] = []

        try:
            # Import a few posts
            posts = [p for p in wxr_data.posts if p.post_type == "post"][:count]
            for post in posts:
                try:
                    # Check if already exists
                    existing = await self._find_by_wp_id("post", post.id)
                    if existing:
                        result.items.append(
                            PreviewItem(
                                entity_type="post",
                                entity_id="",
                                wp_id=post.id,
                                title=post.title,
                                slug=post.slug,
                                status="skip",
                                message="Already exists",
                            )
                        )
                        continue

                    # Sanitize content
                    content_result = self.sanitizer.sanitize(post.content or "")
                    excerpt_result = self.sanitizer.sanitize(post.excerpt or "")

                    # Create entity
                    entity = await self.entity_svc.create(
                        "post",
                        {
                            "title": post.title,
                            "slug": post.slug,
                            "content": content_result.content,
                            "excerpt": excerpt_result.content,
                            "status": WP_STATUS_MAP.get(post.status, "draft"),
                            "wp_id": post.id,
                            "preview": True,  # Mark as preview
                        },
                    )

                    imported_entity_ids.append(entity.id)

                    result.items.append(
                        PreviewItem(
                            entity_type="post",
                            entity_id=entity.id,
                            wp_id=post.id,
                            title=post.title,
                            slug=post.slug,
                            status="success",
                            message="Imported successfully",
                        )
                    )
                    result.success_count += 1

                except Exception as e:
                    result.items.append(
                        PreviewItem(
                            entity_type="post",
                            entity_id="",
                            wp_id=post.id,
                            title=post.title,
                            slug=post.slug,
                            status="error",
                            message=str(e),
                        )
                    )
                    result.error_count += 1
                    result.errors.append(f"Post {post.id}: {e}")

            # Import a few pages if we have room
            remaining = count - len(posts)
            if remaining > 0:
                pages = [p for p in wxr_data.posts if p.post_type == "page"][:remaining]
                for page in pages:
                    try:
                        existing = await self._find_by_wp_id("page", page.id)
                        if existing:
                            continue

                        content_result = self.sanitizer.sanitize(page.content or "")

                        entity = await self.entity_svc.create(
                            "page",
                            {
                                "title": page.title,
                                "slug": page.slug,
                                "content": content_result.content,
                                "status": "draft",
                                "wp_id": page.id,
                                "preview": True,
                            },
                        )

                        imported_entity_ids.append(entity.id)

                        result.items.append(
                            PreviewItem(
                                entity_type="page",
                                entity_id=entity.id,
                                wp_id=page.id,
                                title=page.title,
                                slug=page.slug,
                                status="success",
                                message="Imported successfully",
                            )
                        )
                        result.success_count += 1

                    except Exception as e:
                        result.error_count += 1
                        result.errors.append(f"Page {page.id}: {e}")

            # Store entity IDs for later commit/rollback
            self._preview_entities[preview_id] = imported_entity_ids

            # Commit to save preview items
            await self.db.commit()

            logger.info(
                f"Preview complete: {result.success_count} success, "
                f"{result.error_count} errors"
            )

        except Exception as e:
            logger.error(f"Preview failed: {e}")
            result.errors.append(str(e))
            await self.db.rollback()

        return result

    async def commit_preview(self, preview_id: str) -> dict:
        """
        Commit preview - keep imported items.

        Args:
            preview_id: Preview session ID

        Returns:
            Result dict
        """
        entity_ids = self._preview_entities.get(preview_id, [])

        if not entity_ids:
            return {
                "success": False,
                "error": "Preview not found or already processed",
            }

        # Remove preview flag from entities
        for entity_id in entity_ids:
            result = await self.db.execute(
                select(EntityValue).where(
                    EntityValue.entity_id == entity_id,
                    EntityValue.field_name == "preview",
                )
            )
            ev = result.scalar_one_or_none()
            if ev:
                await self.db.delete(ev)

        await self.db.commit()

        # Clean up
        del self._preview_entities[preview_id]

        logger.info(f"Preview {preview_id} committed: {len(entity_ids)} items")

        return {
            "success": True,
            "preview_id": preview_id,
            "committed_count": len(entity_ids),
        }

    async def rollback_preview(self, preview_id: str) -> dict:
        """
        Rollback preview - delete imported items.

        Args:
            preview_id: Preview session ID

        Returns:
            Result dict
        """
        entity_ids = self._preview_entities.get(preview_id, [])

        if not entity_ids:
            return {
                "success": False,
                "error": "Preview not found or already processed",
            }

        # Delete preview entities
        deleted_count = 0
        for entity_id in entity_ids:
            try:
                result = await self.db.execute(
                    select(Entity).where(Entity.id == entity_id)
                )
                entity = result.scalar_one_or_none()
                if entity:
                    # Soft delete
                    entity.deleted_at = utcnow()
                    deleted_count += 1
            except Exception as e:
                logger.warning(f"Error deleting preview entity {entity_id}: {e}")

        await self.db.commit()

        # Clean up
        del self._preview_entities[preview_id]

        logger.info(f"Preview {preview_id} rolled back: {deleted_count} items deleted")

        return {
            "success": True,
            "preview_id": preview_id,
            "rolled_back_count": deleted_count,
        }

    async def _find_by_wp_id(
        self, entity_type: str, wp_id: int
    ) -> Entity | None:
        """Find entity by WordPress ID."""
        result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == entity_type,
                EntityValue.field_name == "wp_id",
                EntityValue.value_int == wp_id,
                Entity.deleted_at.is_(None),
            )
        )
        return result.scalar_one_or_none()


# Global preview service instance for session management
_preview_sessions: dict[str, list[str]] = {}


def get_preview_entities(preview_id: str) -> list[str]:
    """Get entity IDs for a preview session."""
    return _preview_sessions.get(preview_id, [])


def store_preview_entities(preview_id: str, entity_ids: list[str]) -> None:
    """Store entity IDs for a preview session."""
    _preview_sessions[preview_id] = entity_ids


def clear_preview_entities(preview_id: str) -> None:
    """Clear preview session."""
    if preview_id in _preview_sessions:
        del _preview_sessions[preview_id]



"""Redirect Generator - Creates URL redirects for WordPress migration."""

from dataclasses import dataclass, field
from datetime import datetime, timezone
from urllib.parse import urlparse


@dataclass
class Redirect:
    """Represents a URL redirect."""

    from_path: str
    to_path: str
    status_code: int = 301
    regex: bool = False
    preserve_query: bool = True
    priority: int = 0
    comment: str = ""


@dataclass
class RedirectReport:
    """Report of generated redirects."""

    redirects: list[Redirect] = field(default_factory=list)
    conflicts: list[dict] = field(default_factory=list)
    warnings: list[str] = field(default_factory=list)
    generated_at: datetime = field(default_factory=datetime.utcnow)


class RedirectGenerator:
    """
    Generates URL redirects for WordPress migration.

    Supports:
    - Post/page URL changes
    - Category/tag archive redirects
    - Author archive redirects
    - Date archive redirects
    - Attachment redirects
    - Custom permalink patterns
    - Regex redirects for patterns
    """

    def __init__(
        self,
        old_base_url: str,
        new_base_url: str,
        old_permalink_structure: str = "/%postname%/",
    ):
        """
        Initialize RedirectGenerator.

        Args:
            old_base_url: Old WordPress site URL
            new_base_url: New Focomy site URL
            old_permalink_structure: WordPress permalink structure
        """
        self.old_base = old_base_url.rstrip("/")
        self.new_base = new_base_url.rstrip("/")
        self.permalink_structure = old_permalink_structure

        # Parse old URL
        parsed = urlparse(old_base_url)
        self.old_domain = parsed.netloc

    def generate_from_posts(
        self,
        posts: list[dict],
        new_path_prefix: str = "/blog",
    ) -> list[Redirect]:
        """
        Generate redirects from WordPress posts.

        Args:
            posts: List of post data with old_url, new_slug, post_type
            new_path_prefix: New URL prefix for posts

        Returns:
            List of redirects
        """
        redirects = []

        for post in posts:
            old_url = post.get("old_url", "")
            new_slug = post.get("new_slug", post.get("slug", ""))
            post_type = post.get("post_type", "post")

            if not old_url or not new_slug:
                continue

            # Extract path from old URL
            old_path = self._url_to_path(old_url)
            if not old_path:
                continue

            # Determine new path
            if post_type == "page":
                new_path = f"/page/{new_slug}"
            elif post_type == "attachment":
                # Skip attachments or redirect to media
                continue
            else:
                new_path = f"{new_path_prefix}/{new_slug}"

            redirects.append(
                Redirect(
                    from_path=old_path,
                    to_path=new_path,
                    status_code=301,
                    comment=f"Post: {post.get('title', new_slug)[:50]}",
                )
            )

            # Also handle variations (with/without trailing slash)
            if old_path.endswith("/"):
                redirects.append(
                    Redirect(
                        from_path=old_path.rstrip("/"),
                        to_path=new_path,
                        status_code=301,
                    )
                )
            else:
                redirects.append(
                    Redirect(
                        from_path=old_path + "/",
                        to_path=new_path,
                        status_code=301,
                    )
                )

        return redirects

    def generate_from_categories(
        self,
        categories: list[dict],
        new_path_prefix: str = "/category",
    ) -> list[Redirect]:
        """
        Generate redirects for category archives.

        Args:
            categories: List of category data with old_slug, new_slug
            new_path_prefix: New URL prefix for categories

        Returns:
            List of redirects
        """
        redirects = []

        for cat in categories:
            old_slug = cat.get("old_slug", cat.get("slug", ""))
            new_slug = cat.get("new_slug", old_slug)

            if not old_slug:
                continue

            # WordPress default category URL
            old_path = f"/category/{old_slug}"
            new_path = f"{new_path_prefix}/{new_slug}"

            redirects.append(
                Redirect(
                    from_path=old_path,
                    to_path=new_path,
                    status_code=301,
                    comment=f"Category: {cat.get('name', old_slug)}",
                )
            )

            # With trailing slash
            redirects.append(
                Redirect(
                    from_path=old_path + "/",
                    to_path=new_path,
                    status_code=301,
                )
            )

            # Paginated
            redirects.append(
                Redirect(
                    from_path=f"{old_path}/page/(.+)",
                    to_path=f"{new_path}?page=$1",
                    status_code=301,
                    regex=True,
                )
            )

        return redirects

    def generate_from_tags(
        self,
        tags: list[dict],
        new_path_prefix: str = "/tag",
    ) -> list[Redirect]:
        """
        Generate redirects for tag archives.

        Args:
            tags: List of tag data with old_slug, new_slug
            new_path_prefix: New URL prefix for tags

        Returns:
            List of redirects
        """
        redirects = []

        for tag in tags:
            old_slug = tag.get("old_slug", tag.get("slug", ""))
            new_slug = tag.get("new_slug", old_slug)

            if not old_slug:
                continue

            old_path = f"/tag/{old_slug}"
            new_path = f"{new_path_prefix}/{new_slug}"

            redirects.append(
                Redirect(
                    from_path=old_path,
                    to_path=new_path,
                    status_code=301,
                    comment=f"Tag: {tag.get('name', old_slug)}",
                )
            )

            redirects.append(
                Redirect(
                    from_path=old_path + "/",
                    to_path=new_path,
                    status_code=301,
                )
            )

        return redirects

    def generate_from_authors(
        self,
        authors: list[dict],
        new_path_prefix: str = "/author",
    ) -> list[Redirect]:
        """
        Generate redirects for author archives.

        Args:
            authors: List of author data with login, new_slug
            new_path_prefix: New URL prefix for authors

        Returns:
            List of redirects
        """
        redirects = []

        for author in authors:
            old_login = author.get("login", "")
            new_slug = author.get("new_slug", old_login)

            if not old_login:
                continue

            old_path = f"/author/{old_login}"
            new_path = f"{new_path_prefix}/{new_slug}"

            redirects.append(
                Redirect(
                    from_path=old_path,
                    to_path=new_path,
                    status_code=301,
                    comment=f"Author: {author.get('display_name', old_login)}",
                )
            )

            redirects.append(
                Redirect(
                    from_path=old_path + "/",
                    to_path=new_path,
                    status_code=301,
                )
            )

        return redirects

    def generate_date_archive_redirects(
        self,
        archive_path: str = "/blog/archive",
    ) -> list[Redirect]:
        """
        Generate redirects for date-based archives.

        Args:
            archive_path: New archive path

        Returns:
            List of regex-based redirects
        """
        return [
            # Year archive: /2023/ -> /blog/archive/2023
            Redirect(
                from_path=r"^/(\d{4})/?$",
                to_path=f"{archive_path}/$1",
                status_code=301,
                regex=True,
                comment="Year archive",
            ),
            # Month archive: /2023/01/ -> /blog/archive/2023/01
            Redirect(
                from_path=r"^/(\d{4})/(\d{2})/?$",
                to_path=f"{archive_path}/$1/$2",
                status_code=301,
                regex=True,
                comment="Month archive",
            ),
            # Day archive: /2023/01/15/ -> /blog/archive/2023/01/15
            Redirect(
                from_path=r"^/(\d{4})/(\d{2})/(\d{2})/?$",
                to_path=f"{archive_path}/$1/$2/$3",
                status_code=301,
                regex=True,
                comment="Day archive",
            ),
        ]

    def generate_feed_redirects(
        self,
        new_feed_path: str = "/blog/feed.xml",
    ) -> list[Redirect]:
        """
        Generate redirects for RSS feeds.

        Args:
            new_feed_path: New feed URL

        Returns:
            List of redirects
        """
        return [
            Redirect(from_path="/feed", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/feed/", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/rss", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/rss/", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/atom", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/atom/", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/feed/rss", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/feed/rss2", to_path=new_feed_path, status_code=301),
            Redirect(from_path="/feed/atom", to_path=new_feed_path, status_code=301),
        ]

    def generate_wp_admin_redirects(
        self,
        new_admin_path: str = "/admin",
    ) -> list[Redirect]:
        """
        Generate redirects for wp-admin URLs.

        Args:
            new_admin_path: New admin URL

        Returns:
            List of redirects
        """
        return [
            Redirect(
                from_path="/wp-admin",
                to_path=new_admin_path,
                status_code=301,
                comment="Admin redirect",
            ),
            Redirect(
                from_path="/wp-admin/",
                to_path=new_admin_path,
                status_code=301,
            ),
            Redirect(
                from_path="/wp-login.php",
                to_path=f"{new_admin_path}/login",
                status_code=301,
            ),
            # Block wp-content access
            Redirect(
                from_path="/wp-content/",
                to_path="/",
                status_code=410,  # Gone
                comment="Block old wp-content",
            ),
        ]

    def generate_all(
        self,
        posts: list[dict],
        categories: list[dict],
        tags: list[dict],
        authors: list[dict],
        config: dict | None = None,
    ) -> RedirectReport:
        """
        Generate all redirects.

        Args:
            posts: Post data
            categories: Category data
            tags: Tag data
            authors: Author data
            config: Optional configuration overrides

        Returns:
            RedirectReport with all generated redirects
        """
        config = config or {}
        report = RedirectReport()

        # Collect all redirects
        all_redirects = []

        # Post redirects
        all_redirects.extend(
            self.generate_from_posts(
                posts,
                config.get("post_path_prefix", "/blog"),
            )
        )

        # Category redirects
        all_redirects.extend(
            self.generate_from_categories(
                categories,
                config.get("category_path_prefix", "/category"),
            )
        )

        # Tag redirects
        all_redirects.extend(
            self.generate_from_tags(
                tags,
                config.get("tag_path_prefix", "/tag"),
            )
        )

        # Author redirects
        all_redirects.extend(
            self.generate_from_authors(
                authors,
                config.get("author_path_prefix", "/author"),
            )
        )

        # Date archives
        if config.get("include_date_archives", True):
            all_redirects.extend(
                self.generate_date_archive_redirects(
                    config.get("archive_path", "/blog/archive"),
                )
            )

        # Feeds
        if config.get("include_feeds", True):
            all_redirects.extend(
                self.generate_feed_redirects(
                    config.get("feed_path", "/blog/feed.xml"),
                )
            )

        # Admin redirects
        if config.get("include_admin_redirects", True):
            all_redirects.extend(
                self.generate_wp_admin_redirects(
                    config.get("admin_path", "/admin"),
                )
            )

        # Detect conflicts
        seen_paths = {}
        for redirect in all_redirects:
            path = redirect.from_path
            if path in seen_paths:
                report.conflicts.append(
                    {
                        "path": path,
                        "redirect1": seen_paths[path].to_path,
                        "redirect2": redirect.to_path,
                    }
                )
            else:
                seen_paths[path] = redirect
                report.redirects.append(redirect)

        return report

    def _url_to_path(self, url: str) -> str | None:
        """Extract path from URL."""
        if not url:
            return None

        parsed = urlparse(url)
        path = parsed.path

        # Remove base path if present
        old_parsed = urlparse(self.old_base)
        if old_parsed.path and path.startswith(old_parsed.path):
            path = path[len(old_parsed.path) :]

        return path if path else "/"

    def export_nginx(self, redirects: list[Redirect]) -> str:
        """
        Export redirects as nginx configuration.

        Args:
            redirects: List of redirects

        Returns:
            nginx configuration string
        """
        lines = [
            "# Generated by Focomy WordPress Import",
            f"# Generated at: {datetime.now(timezone.utc).isoformat()}",
            "",
        ]

        # Non-regex redirects first
        for r in redirects:
            if r.regex:
                continue

            if r.status_code == 410:
                lines.append(f"location = {r.from_path} {{ return 410; }}")
            else:
                lines.append(f"location = {r.from_path} {{ return {r.status_code} {r.to_path}; }}")

        lines.append("")
        lines.append("# Regex redirects")

        # Regex redirects
        for r in redirects:
            if not r.regex:
                continue

            if r.status_code == 410:
                lines.append(f"location ~ {r.from_path} {{ return 410; }}")
            else:
                lines.append(f"location ~ {r.from_path} {{ return {r.status_code} {r.to_path}; }}")

        return "\n".join(lines)

    def export_apache(self, redirects: list[Redirect]) -> str:
        """
        Export redirects as Apache .htaccess rules.

        Args:
            redirects: List of redirects

        Returns:
            .htaccess content string
        """
        lines = [
            "# Generated by Focomy WordPress Import",
            f"# Generated at: {datetime.now(timezone.utc).isoformat()}",
            "",
            "RewriteEngine On",
            "",
        ]

        # Non-regex redirects
        for r in redirects:
            if r.regex:
                continue

            if r.status_code == 410:
                lines.append(f"Redirect 410 {r.from_path}")
            else:
                lines.append(f"Redirect {r.status_code} {r.from_path} {r.to_path}")

        lines.append("")
        lines.append("# Regex redirects")

        # Regex redirects
        for r in redirects:
            if not r.regex:
                continue

            if r.status_code == 410:
                lines.append(f"RewriteRule {r.from_path} - [G]")
            else:
                lines.append(f"RewriteRule {r.from_path} {r.to_path} [R={r.status_code},L]")

        return "\n".join(lines)

    def export_json(self, redirects: list[Redirect]) -> str:
        """
        Export redirects as JSON.

        Args:
            redirects: List of redirects

        Returns:
            JSON string
        """
        import json

        data = {
            "generated_at": datetime.now(timezone.utc).isoformat(),
            "redirects": [
                {
                    "from": r.from_path,
                    "to": r.to_path,
                    "status": r.status_code,
                    "regex": r.regex,
                    "comment": r.comment,
                }
                for r in redirects
            ],
        }

        return json.dumps(data, indent=2, ensure_ascii=False)

    def export_yaml(self, redirects: list[Redirect]) -> str:
        """
        Export redirects as YAML (for Focomy config).

        Args:
            redirects: List of redirects

        Returns:
            YAML string
        """
        import yaml

        data = {
            "redirects": [
                {
                    "from": r.from_path,
                    "to": r.to_path,
                    "status": r.status_code,
                    **({"regex": True} if r.regex else {}),
                    **({"comment": r.comment} if r.comment else {}),
                }
                for r in redirects
            ],
        }

        return yaml.dump(data, allow_unicode=True, default_flow_style=False, sort_keys=False)



"""RelationService - relationship management."""

from typing import Any

from sqlalchemy import and_, delete, select
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import Entity, Relation
from .field import FieldService, field_service


class RelationService:
    """
    Relation management service.

    Handles all relationships between entities.
    No foreign keys in entity tables - all managed here.
    """

    def __init__(self, db: AsyncSession, field_svc: FieldService = None):
        self.db = db
        self.field_svc = field_svc or field_service

    async def attach(
        self,
        from_id: str,
        to_id: str,
        relation_type: str,
        sort_order: int = 0,
        metadata: dict[str, Any] = None,
    ) -> Relation:
        """Create a relation between two entities.

        For many_to_one relations, any existing relation from the same
        from_entity is automatically removed (enforcing the constraint).
        """
        # Validate relation type exists
        rel_def = self.field_svc.get_relation_type(relation_type)
        if not rel_def:
            raise ValueError(f"Unknown relation type: {relation_type}")

        # Check if relation already exists
        existing = await self._get_relation(from_id, to_id, relation_type)
        if existing:
            # Update existing
            existing.sort_order = sort_order
            if metadata:
                existing.metadata_ = metadata
            await self.db.commit()
            return existing

        # For many_to_one, remove any existing relation from this from_entity
        # This enforces the "one" constraint
        if rel_def.type == "many_to_one":
            existing_relations = await self.get_relations(from_id, relation_type, direction="from")
            for old_rel in existing_relations:
                await self.db.delete(old_rel)

        # For self_referential relations, check for circular references
        if rel_def.self_referential:
            await self._check_circular_reference(from_id, to_id, relation_type)

        # Create new relation
        relation = Relation(
            from_entity_id=from_id,
            to_entity_id=to_id,
            relation_type=relation_type,
            sort_order=sort_order,
            metadata_=metadata,
        )
        self.db.add(relation)
        await self.db.commit()
        await self.db.refresh(relation)
        return relation

    async def _check_circular_reference(
        self,
        from_id: str,
        to_id: str,
        relation_type: str,
        max_depth: int = 10,
    ) -> None:
        """Check for circular references in self-referential relations.

        Raises:
            ValueError: If adding this relation would create a cycle
        """
        if from_id == to_id:
            raise ValueError("Cannot create self-referential relation to itself")

        # Walk up the tree from to_id to check if we reach from_id
        visited = {to_id}
        current_ids = [to_id]
        depth = 0

        while current_ids and depth < max_depth:
            # Get all parents of current nodes
            parent_ids = []
            for current_id in current_ids:
                relations = await self.get_relations(current_id, relation_type, direction="from")
                for rel in relations:
                    parent_id = rel.to_entity_id
                    if parent_id == from_id:
                        raise ValueError(
                            "Circular reference detected: adding this relation would create a cycle"
                        )
                    if parent_id not in visited:
                        visited.add(parent_id)
                        parent_ids.append(parent_id)

            current_ids = parent_ids
            depth += 1

    async def detach(
        self,
        from_id: str,
        to_id: str,
        relation_type: str,
    ) -> bool:
        """Remove a relation between two entities."""
        relation = await self._get_relation(from_id, to_id, relation_type)
        if not relation:
            return False

        await self.db.delete(relation)
        await self.db.commit()
        return True

    async def sync(
        self,
        from_id: str,
        to_ids: list[str],
        relation_type: str,
    ) -> list[Relation]:
        """
        Sync relations - set exact list of related entities.

        Removes relations not in to_ids, adds missing ones.
        """
        # Get current relations
        current = await self.get_relations(from_id, relation_type)
        current_to_ids = {r.to_entity_id for r in current}
        new_to_ids = set(to_ids)

        # Remove old
        to_remove = current_to_ids - new_to_ids
        for to_id in to_remove:
            await self.detach(from_id, to_id, relation_type)

        # Add new
        to_add = new_to_ids - current_to_ids
        for i, to_id in enumerate(to_ids):
            if to_id in to_add:
                await self.attach(from_id, to_id, relation_type, sort_order=i)

        # Update sort order for existing
        for i, to_id in enumerate(to_ids):
            if to_id in current_to_ids:
                rel = await self._get_relation(from_id, to_id, relation_type)
                if rel:
                    rel.sort_order = i

        await self.db.commit()
        return await self.get_relations(from_id, relation_type)

    async def get_relations(
        self,
        entity_id: str,
        relation_type: str,
        direction: str = "from",
    ) -> list[Relation]:
        """Get all relations of a type for an entity."""
        if direction == "from":
            query = (
                select(Relation)
                .where(
                    and_(
                        Relation.from_entity_id == entity_id,
                        Relation.relation_type == relation_type,
                    )
                )
                .order_by(Relation.sort_order)
            )
        else:
            query = (
                select(Relation)
                .where(
                    and_(
                        Relation.to_entity_id == entity_id,
                        Relation.relation_type == relation_type,
                    )
                )
                .order_by(Relation.sort_order)
            )

        result = await self.db.execute(query)
        return list(result.scalars().all())

    async def get_related(
        self,
        entity_id: str,
        relation_type: str,
        direction: str = "from",
    ) -> list[Entity]:
        """Get related entities."""
        relations = await self.get_relations(entity_id, relation_type, direction)

        if not relations:
            return []

        if direction == "from":
            entity_ids = [r.to_entity_id for r in relations]
        else:
            entity_ids = [r.from_entity_id for r in relations]

        query = select(Entity).where(
            and_(
                Entity.id.in_(entity_ids),
                Entity.deleted_at.is_(None),
            )
        )
        result = await self.db.execute(query)
        entities = {e.id: e for e in result.scalars().all()}

        # Maintain sort order
        return [entities[eid] for eid in entity_ids if eid in entities]

    async def _get_relation(
        self,
        from_id: str,
        to_id: str,
        relation_type: str,
    ) -> Relation | None:
        """Get a specific relation."""
        query = select(Relation).where(
            and_(
                Relation.from_entity_id == from_id,
                Relation.to_entity_id == to_id,
                Relation.relation_type == relation_type,
            )
        )
        result = await self.db.execute(query)
        return result.scalar_one_or_none()

    async def delete_all_relations(
        self,
        entity_id: str,
    ):
        """Delete all relations for an entity (both directions)."""
        await self.db.execute(delete(Relation).where(Relation.from_entity_id == entity_id))
        await self.db.execute(delete(Relation).where(Relation.to_entity_id == entity_id))
        await self.db.commit()



"""Public routes - theme-based content rendering."""

from datetime import datetime, timezone

from fastapi import APIRouter, Depends, HTTPException, Request
from fastapi.responses import HTMLResponse, Response
from sqlalchemy.ext.asyncio import AsyncSession

from ..database import get_db
from ..services.cache import cache_service
from ..services.entity import EntityService, QueryParams
from ..services.field import field_service
from ..services.seo import SEOService
from ..services.theme import theme_service
from ..utils import utcnow
from .helpers import (
    generate_breadcrumbs,
    get_active_theme,
    get_admin_user_optional,
    get_menus_context,
    get_seo_settings,
    get_widgets_context,
    render_theme,
)

router = APIRouter(tags=["public"])


# === Static Assets ===


@router.get("/css/theme.css", response_class=Response)
async def theme_css(db: AsyncSession = Depends(get_db)):
    """Serve theme CSS with caching headers."""
    active_theme = await get_active_theme(db)
    css_content = theme_service.get_css_variables(active_theme)

    return Response(
        content=css_content,
        media_type="text/css",
        headers={
            "Cache-Control": "public, max-age=86400",  # 24 hours
            "Vary": "Accept-Encoding",
        },
    )


# === SEO Routes (must be before dynamic routes) ===


@router.get("/robots.txt", response_class=Response)
async def robots_txt(request: Request):
    """Generate robots.txt dynamically."""
    site_url = str(request.base_url).rstrip("/")

    content = f"""User-agent: *
Allow: /

# Disallow admin and API
Disallow: /admin/
Disallow: /api/

# Sitemap
Sitemap: {site_url}/sitemap.xml
"""
    return Response(content=content, media_type="text/plain")


@router.get("/manifest.json", response_class=Response)
async def manifest_json(request: Request):
    """Generate web app manifest."""
    import json

    site_url = str(request.base_url).rstrip("/")

    manifest = {
        "name": "Focomy",
        "short_name": "Focomy",
        "start_url": "/",
        "display": "standalone",
        "background_color": "#ffffff",
        "theme_color": "#2563eb",
        "icons": [
            {"src": f"{site_url}/static/favicon-192.png", "sizes": "192x192", "type": "image/png"},
            {"src": f"{site_url}/static/favicon-512.png", "sizes": "512x512", "type": "image/png"},
        ],
    }

    return Response(
        content=json.dumps(manifest, ensure_ascii=False, indent=2),
        media_type="application/manifest+json",
    )


@router.get("/sitemap.xml", response_class=Response)
async def sitemap_xml(
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """Generate sitemap.xml dynamically."""
    entity_svc = EntityService(db)
    site_url = str(request.base_url).rstrip("/")
    seo_svc = SEOService(entity_svc, site_url)

    xml_content = await seo_svc.generate_sitemap()

    return Response(content=xml_content, media_type="application/xml")


# === Preview Routes ===


@router.get("/preview/{token}", response_class=HTMLResponse)
async def preview_entity(
    token: str,
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """Preview an entity with a valid token (shows draft/unpublished content)."""
    from ..services.preview import get_preview_service

    preview_svc = get_preview_service(db)
    entity = await preview_svc.get_preview_entity(token)

    if not entity:
        raise HTTPException(status_code=404, detail="Preview not found or expired")

    entity_svc = EntityService(db)
    entity_data = entity_svc.serialize(entity)

    # Get content type info
    content_type = entity.type
    ct = field_service.get_content_type(content_type)

    # Get site URL and contexts
    site_url = str(request.base_url).rstrip("/")
    menus_ctx = await get_menus_context(db)
    widgets_ctx = await get_widgets_context(db)
    seo_ctx = await get_seo_settings(db, site_url)

    # Add preview flag to context
    context = {
        "post": entity_data,
        "entity": entity_data,
        "content": entity_data,
        "is_preview": True,
        "content_type": ct,
        **menus_ctx,
        **widgets_ctx,
        **seo_ctx,
    }

    # Determine template
    template = "post.html"
    if ct and ct.template:
        template = ct.template

    html = await render_theme(db, template, context, request=request)

    return HTMLResponse(content=html)


# Cache TTL settings
PAGE_CACHE_TTL = 300  # 5 minutes for pages
LIST_CACHE_TTL = 60  # 1 minute for listings


@router.api_route("/", methods=["GET", "HEAD"], response_class=HTMLResponse)
async def home(
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """Home page - list recent posts."""
    cache_key = "page:home"
    admin_info = await get_admin_user_optional(request, db)
    if not admin_info:
        cached_html = await cache_service.get(cache_key)
        if cached_html:
            return HTMLResponse(content=cached_html)

    entity_svc = EntityService(db)

    posts = await entity_svc.find(
        "post",
        limit=10,
        order_by="-created_at",
        filters={"status": "published"},
    )

    # Filter by published_at (scheduled posts)
    now = utcnow()
    posts_data = []
    for p in posts:
        data = entity_svc.serialize(p)
        published_at = data.get("published_at")
        # Show if no published_at set or if it's in the past
        if not published_at or datetime.fromisoformat(published_at.replace("Z", "")) <= now:
            posts_data.append(data)

    # Get menus, widgets, and SEO settings
    site_url = str(request.base_url).rstrip("/")
    menus_ctx = await get_menus_context(db)
    widgets_ctx = await get_widgets_context(db)
    seo_ctx = await get_seo_settings(db, site_url)

    html = await render_theme(
        db,
        "home.html",
        {
            "posts": posts_data,
            **menus_ctx,
            **widgets_ctx,
            **seo_ctx,
        },
        request=request,
    )

    await cache_service.set(cache_key, html, LIST_CACHE_TTL)
    return HTMLResponse(content=html)


@router.get("/page/{slug}", response_class=HTMLResponse)
async def view_page(
    slug: str,
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """View a page."""
    entity_svc = EntityService(db)

    pages = await entity_svc.find(
        "page",
        limit=1,
        filters={"slug": slug},
    )

    if not pages:
        raise HTTPException(status_code=404, detail="Page not found")

    page = pages[0]
    page_data = entity_svc.serialize(page)

    # Get menus and SEO settings first
    site_url = str(request.base_url).rstrip("/")
    menus_ctx = await get_menus_context(db)
    seo_ctx = await get_seo_settings(db, site_url)

    # Generate SEO meta with site settings
    seo_svc = SEOService(entity_svc, site_url, seo_ctx.get("_seo_service_settings", {}))
    meta = seo_svc.generate_meta(page)
    seo_meta = seo_svc.render_meta_tags(meta)

    # Generate breadcrumbs
    breadcrumb_ctx = generate_breadcrumbs(
        [{"name": page_data.get("title", "ページ"), "url": f"/page/{slug}"}], site_url
    )

    html = await render_theme(
        db,
        "post.html",
        {
            "post": page_data,
            "seo_meta": seo_meta,
            **menus_ctx,
            **seo_ctx,
            **breadcrumb_ctx,
        },
        request=request,
        entity=page,
        content_type="page",
    )

    return HTMLResponse(content=html)


# === Category ===


@router.get("/category/{slug}", response_class=HTMLResponse)
async def view_category(
    slug: str,
    request: Request,
    page: int = 1,
    db: AsyncSession = Depends(get_db),
):
    """View posts in a category."""
    entity_svc = EntityService(db)

    # Find category
    categories = await entity_svc.find("category", limit=1, filters={"slug": slug})
    if not categories:
        raise HTTPException(status_code=404, detail="Category not found")

    category = categories[0]
    category_data = entity_svc.serialize(category)

    # Find posts in category (via relation)
    from ..services.relation import RelationService

    relation_svc = RelationService(db)

    per_page = 10
    offset = (page - 1) * per_page

    # Get related posts
    related = await relation_svc.get_related(category.id, "post_categories", reverse=True)
    post_ids = [e.id for e in related]

    # Filter published posts
    posts = []
    for pid in post_ids[offset : offset + per_page]:
        post = await entity_svc.get(pid)
        if post:
            post_data = entity_svc.serialize(post)
            if post_data.get("status") == "published":
                posts.append(post_data)

    total = len(list(post_ids))  # Simplified count
    total_pages = (total + per_page - 1) // per_page

    # Get menus and site context
    site_url = str(request.base_url).rstrip("/")
    menus_ctx = await get_menus_context(db)
    seo_ctx = await get_seo_settings(db, site_url)

    html = await render_theme(
        db,
        "category.html",
        {
            "category": category_data,
            "posts": posts,
            "page": page,
            "total_pages": total_pages,
            **menus_ctx,
            **seo_ctx,
        },
        request=request,
        entity=category,
        content_type="category",
    )

    return HTMLResponse(content=html)


# === Archive ===


@router.get("/archive/{year}/{month}", response_class=HTMLResponse)
async def view_archive(
    year: int,
    month: int,
    request: Request,
    page: int = 1,
    db: AsyncSession = Depends(get_db),
):
    """View posts from a specific month."""
    entity_svc = EntityService(db)

    # Calculate date range
    from datetime import date

    date(year, month, 1)
    if month == 12:
        date(year + 1, 1, 1)
    else:
        date(year, month + 1, 1)

    per_page = 10
    offset = (page - 1) * per_page

    # Find posts in date range
    posts_raw = await entity_svc.find(
        "post",
        limit=per_page,
        offset=offset,
        order_by="-created_at",
        filters={"status": "published"},
    )

    # Filter by date (in-memory for now, ideally use DB filter)
    posts = []
    for p in posts_raw:
        data = entity_svc.serialize(p)
        created = data.get("created_at", "")
        if created and created[:7] == f"{year:04d}-{month:02d}":
            posts.append(data)

    # Get menus and site context
    site_url = str(request.base_url).rstrip("/")
    menus_ctx = await get_menus_context(db)
    seo_ctx = await get_seo_settings(db, site_url)

    html = await render_theme(
        db,
        "archive.html",
        {
            "year": year,
            "month": month,
            "posts": posts,
            "page": page,
            **menus_ctx,
            **seo_ctx,
        },
        request=request,
    )

    return HTMLResponse(content=html)


# === Search ===


@router.get("/search", response_class=HTMLResponse)
async def search(
    request: Request,
    q: str = "",
    page: int = 1,
    db: AsyncSession = Depends(get_db),
):
    """Search posts."""
    entity_svc = EntityService(db)

    posts = []
    total = 0

    if q:
        per_page = 10
        offset = (page - 1) * per_page

        # Search in posts
        all_posts = await entity_svc.find(
            "post",
            limit=100,
            order_by="-created_at",
            filters={"status": "published"},
        )

        # Simple in-memory search
        q_lower = q.lower()
        matched = []
        for p in all_posts:
            data = entity_svc.serialize(p)
            title = str(data.get("title", "")).lower()
            excerpt = str(data.get("excerpt", "")).lower()
            if q_lower in title or q_lower in excerpt:
                matched.append(data)

        total = len(matched)
        posts = matched[offset : offset + per_page]

    total_pages = (total + 9) // 10 if total > 0 else 0

    # Get menus and site context
    site_url = str(request.base_url).rstrip("/")
    menus_ctx = await get_menus_context(db)
    seo_ctx = await get_seo_settings(db, site_url)

    html = await render_theme(
        db,
        "search.html",
        {
            "query": q,
            "posts": posts,
            "total": total,
            "page": page,
            "total_pages": total_pages,
            **menus_ctx,
            **seo_ctx,
        },
        request=request,
    )

    return HTMLResponse(content=html)


# === Include sub-routers ===
# Order matters: specific routes first, catch-all routes last

from .routes_feeds import router as feeds_router
from .routes_channels import router as channels_router
from .routes_dynamic import router as dynamic_router

router.include_router(feeds_router)
router.include_router(channels_router)
router.include_router(dynamic_router)  # Must be last (has catch-all patterns)



"""Preview Service - Generate preview tokens for draft content."""

import secrets
from datetime import datetime, timedelta, timezone

from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import Entity
from ..utils import utcnow

# In-memory preview token storage (for simplicity)
# In production, consider using Redis or database
_preview_tokens: dict[str, dict] = {}


class PreviewService:
    """
    Service for generating preview URLs for draft content.

    Usage:
        # Generate preview token
        token = await preview_svc.create_token(entity_id, user_id)
        preview_url = f"/preview/{token}"

        # Verify token and get entity
        entity = await preview_svc.get_preview_entity(token)
    """

    TOKEN_EXPIRY_HOURS = 24
    TOKEN_LENGTH = 32

    def __init__(self, db: AsyncSession):
        self.db = db

    async def create_token(
        self,
        entity_id: str,
        user_id: str,
        expiry_hours: int | None = None,
    ) -> str:
        """
        Create a preview token for an entity.

        Args:
            entity_id: The entity to preview
            user_id: The user creating the preview
            expiry_hours: Custom expiry (default 24 hours)

        Returns:
            Preview token string
        """
        expiry = expiry_hours or self.TOKEN_EXPIRY_HOURS
        token = secrets.token_urlsafe(self.TOKEN_LENGTH)

        _preview_tokens[token] = {
            "entity_id": entity_id,
            "user_id": user_id,
            "created_at": utcnow(),
            "expires_at": utcnow() + timedelta(hours=expiry),
        }

        # Clean up expired tokens periodically
        self._cleanup_expired_tokens()

        return token

    async def get_preview_entity(self, token: str) -> Entity | None:
        """
        Get entity for preview if token is valid.

        Returns:
            Entity if token is valid, None otherwise
        """
        token_data = _preview_tokens.get(token)
        if not token_data:
            return None

        # Check expiration
        if utcnow() > token_data["expires_at"]:
            del _preview_tokens[token]
            return None

        # Get entity (including drafts/deleted)
        entity_id = token_data["entity_id"]
        result = await self.db.execute(select(Entity).where(Entity.id == entity_id))
        return result.scalar_one_or_none()

    async def revoke_token(self, token: str) -> bool:
        """Revoke a preview token."""
        if token in _preview_tokens:
            del _preview_tokens[token]
            return True
        return False

    async def revoke_entity_tokens(self, entity_id: str) -> int:
        """Revoke all preview tokens for an entity."""
        count = 0
        tokens_to_delete = [
            t for t, data in _preview_tokens.items() if data["entity_id"] == entity_id
        ]
        for token in tokens_to_delete:
            del _preview_tokens[token]
            count += 1
        return count

    async def get_entity_tokens(self, entity_id: str) -> list[dict]:
        """Get all active preview tokens for an entity."""
        self._cleanup_expired_tokens()
        return [
            {
                "token": token,
                "created_at": data["created_at"],
                "expires_at": data["expires_at"],
            }
            for token, data in _preview_tokens.items()
            if data["entity_id"] == entity_id
        ]

    def _cleanup_expired_tokens(self):
        """Remove expired tokens."""
        now = utcnow()
        expired = [t for t, data in _preview_tokens.items() if now > data["expires_at"]]
        for token in expired:
            del _preview_tokens[token]

    def get_preview_url(self, token: str, base_url: str = "") -> str:
        """Generate full preview URL."""
        return f"{base_url}/preview/{token}"


# Preview routes helper
def get_preview_service(db: AsyncSession) -> PreviewService:
    return PreviewService(db)



"""Feed routes - RSS, Atom, and JSON feeds."""

import json as json_mod
from datetime import datetime, timezone

from fastapi import APIRouter, Depends, HTTPException, Request
from fastapi.responses import Response
from sqlalchemy.ext.asyncio import AsyncSession

from ..database import get_db
from ..services.entity import EntityService, QueryParams
from ..services.field import field_service
from ..services.settings import SettingsService
from ..services.theme import theme_service
from ..utils import utcnow

router = APIRouter(tags=["feeds"])


@router.get("/feed.xml", response_class=Response)
async def rss_feed(
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """RSS feed for posts (default)."""
    return await _generate_rss_feed("post", request, db)


@router.get("/atom.xml", response_class=Response)
async def atom_feed(
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """Atom feed for posts."""
    return await _generate_atom_feed("post", request, db)


@router.get("/feed.json", response_class=Response)
async def json_feed(
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """JSON Feed for posts."""
    return await _generate_json_feed("post", request, db)


@router.get("/{path_prefix}/feed.xml", response_class=Response)
async def content_type_feed(
    path_prefix: str,
    request: Request,
    db: AsyncSession = Depends(get_db),
):
    """RSS feed for content types with feed_enabled."""
    content_types = field_service.get_all_content_types()
    target_ct = None

    for _ct_name, ct in content_types.items():
        prefix = ct.path_prefix.strip("/")
        if prefix and prefix == path_prefix and ct.feed_enabled:
            target_ct = ct
            break

    if not target_ct:
        raise HTTPException(status_code=404, detail="Feed not found")

    return await _generate_rss_feed(target_ct.name, request, db)


def _format_rfc822(iso_date: str) -> str:
    """Convert ISO date string to RFC822 format for RSS feeds."""
    if not iso_date:
        return ""
    try:
        # Handle various ISO formats
        dt = datetime.fromisoformat(iso_date.replace("Z", "+00:00"))
        return dt.strftime("%a, %d %b %Y %H:%M:%S +0000")
    except (ValueError, AttributeError):
        return ""


def _build_feed_url(site_url: str, path_prefix: str, slug: str) -> str:
    """Build URL for feed items, avoiding double slashes."""
    parts = [p for p in [site_url.rstrip("/"), path_prefix.strip("/"), slug.strip("/")] if p]
    return "/".join(parts)


async def _get_feed_entities(content_type: str, db: AsyncSession) -> tuple:
    """Get entities for feed generation."""
    entity_svc = EntityService(db)
    ct = field_service.get_content_type(content_type)

    entities = await entity_svc.find(
        content_type,
        limit=20,
        order_by="-created_at",
        filters={"status": "published"},
    )

    return entity_svc, ct, entities


async def _generate_rss_feed(
    content_type: str,
    request: Request,
    db: AsyncSession,
) -> Response:
    """Generate RSS 2.0 feed."""
    entity_svc, ct, entities = await _get_feed_entities(content_type, db)
    site_url = str(request.base_url).rstrip("/")
    path_prefix = ct.path_prefix.strip("/") if ct else content_type

    items = []
    for e in entities:
        data = entity_svc.serialize(e)
        slug = data.get("slug", "")
        url = _build_feed_url(site_url, path_prefix, slug)
        pub_date = _format_rfc822(data.get("created_at", ""))
        items.append(
            f"""
        <item>
            <title><![CDATA[{data.get('title', '')}]]></title>
            <link>{url}</link>
            <description><![CDATA[{data.get('excerpt', '')}]]></description>
            <pubDate>{pub_date}</pubDate>
            <guid>{url}</guid>
        </item>"""
        )

    title = ct.label_plural if ct else "Posts"
    rss = f"""<?xml version="1.0" encoding="UTF-8"?>
<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">
    <channel>
        <title>{title}</title>
        <link>{site_url}</link>
        <description>Latest {title.lower()}</description>
        <language>ja</language>
        <atom:link href="{site_url}/feed.xml" rel="self" type="application/rss+xml"/>
        {''.join(items)}
    </channel>
</rss>"""

    return Response(content=rss, media_type="application/rss+xml")


async def _generate_atom_feed(
    content_type: str,
    request: Request,
    db: AsyncSession,
) -> Response:
    """Generate Atom feed."""
    entity_svc, ct, entities = await _get_feed_entities(content_type, db)
    site_url = str(request.base_url).rstrip("/")
    path_prefix = ct.path_prefix.strip("/") if ct else content_type

    entries = []
    for e in entities:
        data = entity_svc.serialize(e)
        slug = data.get("slug", "")
        url = _build_feed_url(site_url, path_prefix, slug)
        entries.append(
            f"""
    <entry>
        <title><![CDATA[{data.get('title', '')}]]></title>
        <link href="{url}"/>
        <id>{url}</id>
        <updated>{data.get('updated_at', data.get('created_at', ''))}</updated>
        <summary><![CDATA[{data.get('excerpt', '')}]]></summary>
    </entry>"""
        )

    title = ct.label_plural if ct else "Posts"
    updated = entities[0].updated_at.isoformat() if entities else utcnow().isoformat()

    atom = f"""<?xml version="1.0" encoding="UTF-8"?>
<feed xmlns="http://www.w3.org/2005/Atom">
    <title>{title}</title>
    <link href="{site_url}"/>
    <link href="{site_url}/atom.xml" rel="self"/>
    <id>{site_url}/</id>
    <updated>{updated}</updated>
    {''.join(entries)}
</feed>"""

    return Response(content=atom, media_type="application/atom+xml")


async def _generate_json_feed(
    content_type: str,
    request: Request,
    db: AsyncSession,
) -> Response:
    """Generate JSON Feed (https://jsonfeed.org/)."""
    import json

    entity_svc, ct, entities = await _get_feed_entities(content_type, db)
    site_url = str(request.base_url).rstrip("/")
    path_prefix = ct.path_prefix.strip("/") if ct else content_type

    items = []
    for e in entities:
        data = entity_svc.serialize(e)
        slug = data.get("slug", "")
        url = _build_feed_url(site_url, path_prefix, slug)
        items.append(
            {
                "id": url,
                "url": url,
                "title": data.get("title", ""),
                "content_text": data.get("excerpt", ""),
                "date_published": data.get("created_at", ""),
                "date_modified": data.get("updated_at", data.get("created_at", "")),
            }
        )

    title = ct.label_plural if ct else "Posts"
    feed = {
        "version": "https://jsonfeed.org/version/1.1",
        "title": title,
        "home_page_url": site_url,
        "feed_url": f"{site_url}/feed.json",
        "language": "ja",
        "items": items,
    }

    return Response(
        content=json.dumps(feed, ensure_ascii=False, indent=2), media_type="application/feed+json"
    )



"""EntityService - unified CRUD for all content types."""

from dataclasses import dataclass, field
from datetime import datetime, timedelta, timezone
from typing import Any

from sqlalchemy import and_, func, select
from sqlalchemy.ext.asyncio import AsyncSession

from ..models import Entity, EntityValue
from ..utils import utcnow
from .cache import cache_service
from .field import FieldService, field_service


@dataclass
class QueryParams:
    """Query parameters for find operations."""

    filters: dict[str, Any] = field(default_factory=dict)
    sort: str = "-created_at"
    page: int = 1
    per_page: int = 20
    include_deleted: bool = False
    search: str = ""  # Full-text search query


class EntityService:
    """
    Unified CRUD service for all entity types.

    No PostService, PageService, etc.
    This single service handles all content types.
    """

    def __init__(self, db: AsyncSession, field_svc: FieldService = None):
        self.db = db
        self.field_svc = field_svc or field_service

    async def create(
        self,
        type_name: str,
        data: dict[str, Any],
        user_id: str = None,
    ) -> Entity:
        """Create a new entity.

        All operations are performed in a single transaction.
        On error, the entire transaction is rolled back.
        """
        # Validate before starting transaction
        validation = self.field_svc.validate(type_name, data)
        if not validation.valid:
            raise ValueError(f"Validation failed: {validation.errors}")

        # Check unique constraints before creating
        ct = self.field_svc.get_content_type(type_name)
        if ct:
            await self._check_unique_constraints(type_name, data, ct, entity_id=None)

        try:
            # Create entity
            entity = Entity(
                type=type_name,
                created_by=user_id,
                updated_by=user_id,
            )
            self.db.add(entity)
            await self.db.flush()

            # Create values
            if ct:
                for field_def in ct.fields:
                    value = data.get(field_def.name, field_def.default)
                    if value is not None:
                        await self._set_value(entity.id, field_def.name, value, field_def.type)

            await self.db.commit()

            # Invalidate cache for this content type
            await self._invalidate_cache(type_name)

            # Re-fetch to ensure values relationship is loaded
            return await self.get(entity.id)

        except Exception:
            await self.db.rollback()
            raise

    async def update(
        self,
        entity_id: str,
        data: dict[str, Any],
        user_id: str = None,
        create_revision: bool = True,
        revision_type: str = "manual",
        expected_version: int = None,
    ) -> Entity | None:
        """Update an existing entity.

        All operations are performed in a single transaction.
        On error, the entire transaction is rolled back.

        Args:
            expected_version: If provided, checks for concurrent modification.
                              Raises ValueError if versions don't match.
        """
        entity = await self.get(entity_id)
        if not entity:
            return None

        # Optimistic locking: check version if expected_version is provided
        if expected_version is not None and entity.version != expected_version:
            raise ValueError(
                f"Concurrent modification detected. Expected version {expected_version}, "
                f"but current version is {entity.version}. Please reload and try again."
            )

        # Validate before starting transaction
        validation = self.field_svc.validate(entity.type, data)
        if not validation.valid:
            raise ValueError(f"Validation failed: {validation.errors}")

        # Check unique constraints and transitions before updating
        ct = self.field_svc.get_content_type(entity.type)
        if ct:
            await self._check_unique_constraints(entity.type, data, ct, entity_id=entity_id)
            await self._check_status_transitions(entity, data, ct)

        try:
            # Create revision before updating
            if create_revision:
                from .revision import RevisionService

                revision_svc = RevisionService(self.db)
                current_data = self.serialize(entity)
                await revision_svc.create(
                    entity_id=entity_id,
                    data=current_data,
                    revision_type=revision_type,
                    title=current_data.get("title") or current_data.get("name"),
                    user_id=user_id,
                )

            # Update entity
            entity.updated_at = utcnow()
            entity.updated_by = user_id
            entity.version += 1  # Increment version for optimistic locking

            # Update values
            if ct:
                for field_def in ct.fields:
                    if field_def.name in data:
                        await self._set_value(
                            entity_id,
                            field_def.name,
                            data[field_def.name],
                            field_def.type,
                        )

            await self.db.commit()

            # Invalidate cache for this content type
            await self._invalidate_cache(entity.type)

            # Re-fetch to ensure values relationship is loaded
            return await self.get(entity_id)

        except Exception:
            await self.db.rollback()
            raise

    async def delete(
        self,
        entity_id: str,
        user_id: str = None,
        hard: bool = False,
        cascade: bool = True,
    ) -> bool:
        """Delete an entity (soft delete by default).

        Args:
            entity_id: Entity to delete
            user_id: User performing the delete
            hard: If True, permanently delete (not recommended)
            cascade: If True, also soft-delete related entities marked with cascade_delete

        Returns:
            True if deleted, False if entity not found
        """
        entity = await self.get(entity_id, include_deleted=True)
        if not entity:
            return False

        entity_type = entity.type
        deleted_at = utcnow()

        try:
            if hard:
                await self.db.delete(entity)
            else:
                entity.deleted_at = deleted_at
                entity.updated_by = user_id

                # Handle cascade deletes
                if cascade:
                    await self._cascade_soft_delete(
                        entity_id=entity_id,
                        entity_type=entity_type,
                        user_id=user_id,
                        deleted_at=deleted_at,
                    )

            await self.db.commit()

            # Invalidate cache for this content type
            await self._invalidate_cache(entity_type)

            return True

        except Exception:
            await self.db.rollback()
            raise

    async def _cascade_soft_delete(
        self,
        entity_id: str,
        entity_type: str,
        user_id: str,
        deleted_at: datetime,
    ) -> int:
        """Cascade soft delete to related entities.

        Finds all relations with cascade_delete=True that point TO this entity
        and soft-deletes the from_entities.

        Returns:
            Count of entities that were cascade-deleted
        """
        from ..models import Relation

        cascade_relations = self.field_svc.get_cascade_relations_for_type(entity_type)
        if not cascade_relations:
            return 0

        count = 0
        for relation_name, _rel_def in cascade_relations:
            # Find all entities that have a relation TO the deleted entity
            query = select(Relation).where(
                and_(
                    Relation.to_entity_id == entity_id,
                    Relation.relation_type == relation_name,
                )
            )
            result = await self.db.execute(query)
            relations = result.scalars().all()

            for relation in relations:
                # Get the from_entity and soft delete it
                from_entity = await self.get(relation.from_entity_id)
                if from_entity and from_entity.deleted_at is None:
                    from_entity.deleted_at = deleted_at
                    from_entity.updated_by = user_id
                    count += 1

                    # Recursively cascade (with depth limit handled by DB)
                    count += await self._cascade_soft_delete(
                        entity_id=from_entity.id,
                        entity_type=from_entity.type,
                        user_id=user_id,
                        deleted_at=deleted_at,
                    )

        return count

    async def restore(
        self,
        entity_id: str,
        user_id: str = None,
    ) -> Entity | None:
        """
        Restore a soft-deleted entity.

        Args:
            entity_id: Entity to restore
            user_id: User performing the restore

        Returns:
            Restored entity, or None if not found
        """
        entity = await self.get(entity_id, include_deleted=True)
        if not entity or entity.deleted_at is None:
            return None

        entity.deleted_at = None
        entity.updated_at = utcnow()
        entity.updated_by = user_id

        await self.db.commit()

        await self._invalidate_cache(entity.type)
        return entity

    async def list_deleted(
        self,
        type_name: str = None,
        limit: int = 50,
        offset: int = 0,
    ) -> tuple[list[Entity], int]:
        """
        List soft-deleted entities (trash).

        Args:
            type_name: Optional filter by type
            limit: Max results
            offset: Pagination offset

        Returns:
            Tuple of (entities, total_count)
        """
        query = select(Entity).where(Entity.deleted_at.isnot(None))

        if type_name:
            query = query.where(Entity.type == type_name)

        # Order by deletion date (most recent first)
        query = query.order_by(Entity.deleted_at.desc())

        # Get total count
        from sqlalchemy import func

        count_query = select(func.count(Entity.id)).where(Entity.deleted_at.isnot(None))
        if type_name:
            count_query = count_query.where(Entity.type == type_name)
        count_result = await self.db.execute(count_query)
        total = count_result.scalar() or 0

        # Get entities with pagination
        query = query.limit(limit).offset(offset)
        result = await self.db.execute(query)
        entities = result.scalars().all()

        return list(entities), total

    async def empty_trash(
        self,
        type_name: str = None,
        older_than_days: int = 30,
    ) -> int:
        """
        Permanently delete old soft-deleted entities.

        Args:
            type_name: Optional filter by type
            older_than_days: Only delete items deleted more than X days ago

        Returns:
            Count of permanently deleted entities
        """
        from sqlalchemy import delete as sql_delete

        cutoff = utcnow() - timedelta(days=older_than_days)

        conditions = [
            Entity.deleted_at.isnot(None),
            Entity.deleted_at < cutoff,
        ]
        if type_name:
            conditions.append(Entity.type == type_name)

        result = await self.db.execute(sql_delete(Entity).where(and_(*conditions)))
        await self.db.commit()

        return result.rowcount

    async def _invalidate_cache(self, type_name: str) -> None:
        """Invalidate page cache for a content type."""
        # Invalidate home page (shows recent posts)
        await cache_service.delete("page:home")

        # Invalidate type-specific pages
        await cache_service.invalidate_pattern(f"page:{type_name}")

        # Invalidate listings
        await cache_service.invalidate_pattern("page:category")
        await cache_service.invalidate_pattern("page:archive")

    async def get(
        self,
        entity_id: str,
        include_deleted: bool = False,
    ) -> Entity | None:
        """Get entity by ID."""
        query = select(Entity).where(Entity.id == entity_id)
        if not include_deleted:
            query = query.where(Entity.deleted_at.is_(None))

        result = await self.db.execute(query)
        return result.scalar_one_or_none()

    async def find(
        self,
        type_name: str,
        params: QueryParams = None,
        *,
        limit: int = None,
        offset: int = None,
        order_by: str = None,
        filters: dict = None,
        include_deleted: bool = False,
    ) -> list[Entity]:
        """Find entities by type with filters."""
        # Support both QueryParams and keyword arguments
        if params:
            _filters = params.filters
            _sort = params.sort
            _offset = (params.page - 1) * params.per_page
            _limit = params.per_page
            _include_deleted = params.include_deleted
        else:
            _filters = filters or {}
            _sort = order_by or "-created_at"
            _offset = offset or 0
            _limit = limit or 20
            _include_deleted = include_deleted

        query = select(Entity).where(Entity.type == type_name)

        if not _include_deleted:
            query = query.where(Entity.deleted_at.is_(None))

        # Apply filters
        for field_name, value in _filters.items():
            if isinstance(value, dict):
                # Complex filter (gte, lte, etc.)
                for op, val in value.items():
                    query = self._apply_filter(query, field_name, op, val)
            else:
                # Simple equality
                query = self._apply_filter(query, field_name, "eq", value)

        # Apply sort
        if _sort.startswith("-"):
            sort_field = _sort[1:]
            desc = True
        else:
            sort_field = _sort
            desc = False

        if sort_field in ("created_at", "updated_at"):
            order_col = getattr(Entity, sort_field)
            query = query.order_by(order_col.desc() if desc else order_col)

        # Apply pagination
        query = query.offset(_offset).limit(_limit)

        result = await self.db.execute(query)
        return list(result.scalars().all())

    async def count(
        self,
        type_name: str,
        params: QueryParams = None,
    ) -> int:
        """Count entities by type with filters."""
        params = params or QueryParams()

        query = select(func.count()).select_from(Entity).where(Entity.type == type_name)

        if not params.include_deleted:
            query = query.where(Entity.deleted_at.is_(None))

        for field_name, value in params.filters.items():
            if isinstance(value, dict):
                for op, val in value.items():
                    query = self._apply_filter(query, field_name, op, val)
            else:
                query = self._apply_filter(query, field_name, "eq", value)

        result = await self.db.execute(query)
        return result.scalar() or 0

    async def _set_value(
        self,
        entity_id: str,
        field_name: str,
        value: Any,
        field_type: str,
    ):
        """Set a field value on an entity."""
        # Check if value exists
        query = select(EntityValue).where(
            and_(
                EntityValue.entity_id == entity_id,
                EntityValue.field_name == field_name,
            )
        )
        result = await self.db.execute(query)
        ev = result.scalar_one_or_none()

        if ev is None:
            ev = EntityValue(entity_id=entity_id, field_name=field_name)
            self.db.add(ev)

        # Clear all value columns
        ev.value_text = None
        ev.value_int = None
        ev.value_float = None
        ev.value_datetime = None
        ev.value_json = None

        # Set appropriate column based on type
        storage_type = self._get_storage_type(field_type)
        if storage_type == "text":
            ev.value_text = str(value) if value is not None else None
        elif storage_type == "int":
            ev.value_int = int(value) if value is not None else None
        elif storage_type == "float":
            ev.value_float = float(value) if value is not None else None
        elif storage_type == "datetime":
            if isinstance(value, datetime):
                ev.value_datetime = value
            elif isinstance(value, str):
                ev.value_datetime = datetime.fromisoformat(value)
        elif storage_type == "json":
            ev.value_json = value

    def _get_storage_type(self, field_type: str) -> str:
        """Get storage type for a field type."""
        type_mapping = {
            "string": "text",
            "text": "text",
            "slug": "text",
            "email": "text",
            "url": "text",
            "password": "text",
            "integer": "int",
            "float": "float",
            "boolean": "int",
            "datetime": "datetime",
            "date": "datetime",
            "select": "text",
            "multiselect": "json",
            "blocks": "json",
            "media": "text",
            "json": "json",
        }
        return type_mapping.get(field_type, "text")

    def _apply_filter(self, query, field_name: str, op: str, value: Any):
        """Apply a filter to the query."""
        # Entity-level fields
        if field_name in ("created_at", "updated_at", "deleted_at"):
            col = getattr(Entity, field_name)
            if op == "eq":
                return query.where(col == value)
            elif op == "gte":
                return query.where(col >= value)
            elif op == "lte":
                return query.where(col <= value)
            elif op == "gt":
                return query.where(col > value)
            elif op == "lt":
                return query.where(col < value)
            return query

        # EntityValue fields - use subquery
        # Determine which value column to compare based on value type
        if isinstance(value, bool):
            value_col = EntityValue.value_int
            compare_value = 1 if value else 0
        elif isinstance(value, int):
            value_col = EntityValue.value_int
            compare_value = value
        elif isinstance(value, float):
            value_col = EntityValue.value_float
            compare_value = value
        elif isinstance(value, datetime):
            value_col = EntityValue.value_datetime
            compare_value = value
        else:
            # Default to text comparison
            value_col = EntityValue.value_text
            compare_value = str(value) if value is not None else None

        # Build subquery for EntityValue filtering
        if op == "eq":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col == compare_value)
            )
        elif op == "neq":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col != compare_value)
            )
        elif op == "gte":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col >= compare_value)
            )
        elif op == "lte":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col <= compare_value)
            )
        elif op == "gt":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col > compare_value)
            )
        elif op == "lt":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col < compare_value)
            )
        elif op == "like":
            subq = select(EntityValue.entity_id).where(
                and_(EntityValue.field_name == field_name, value_col.like(f"%{compare_value}%"))
            )
        elif op == "isnull":
            # Check if field doesn't exist or value is null
            if value:
                subq = select(EntityValue.entity_id).where(
                    and_(EntityValue.field_name == field_name, value_col.is_(None))
                )
            else:
                subq = select(EntityValue.entity_id).where(
                    and_(EntityValue.field_name == field_name, value_col.isnot(None))
                )
        else:
            return query

        return query.where(Entity.id.in_(subq))

    async def _check_unique_constraints(
        self,
        type_name: str,
        data: dict[str, Any],
        content_type,
        entity_id: str = None,
    ) -> None:
        """Check unique constraints for fields.

        Args:
            type_name: Content type name
            data: Data to check
            content_type: ContentType definition
            entity_id: Current entity ID (for updates, to exclude self)

        Raises:
            ValueError: If a unique constraint is violated
        """
        for field_def in content_type.fields:
            if not field_def.unique:
                continue

            value = data.get(field_def.name)
            if value is None:
                continue

            # Check if another entity has this value
            storage_type = self._get_storage_type(field_def.type)
            if storage_type == "text":
                value_col = EntityValue.value_text
                compare_value = str(value)
            elif storage_type == "int":
                value_col = EntityValue.value_int
                compare_value = int(value)
            else:
                continue  # Skip non-text/int unique checks for now

            # Build query to find existing entity with same value
            subq = select(EntityValue.entity_id).where(
                and_(
                    EntityValue.field_name == field_def.name,
                    value_col == compare_value,
                )
            )

            query = select(Entity).where(
                and_(
                    Entity.type == type_name,
                    Entity.deleted_at.is_(None),
                    Entity.id.in_(subq),
                )
            )

            # Exclude current entity for updates
            if entity_id:
                query = query.where(Entity.id != entity_id)

            result = await self.db.execute(query)
            existing = result.scalar_one_or_none()

            if existing:
                raise ValueError(
                    f"Unique constraint violation: {field_def.label or field_def.name} "
                    f"'{value}' already exists"
                )

    async def _check_status_transitions(
        self,
        entity: Entity,
        data: dict[str, Any],
        content_type,
    ) -> None:
        """Check if status field transitions are valid.

        Args:
            entity: Entity being updated
            data: New data
            content_type: ContentType definition

        Raises:
            ValueError: If a status transition is not allowed
        """
        # Get current field values for comparison
        current_data = self.serialize(entity)

        for field_def in content_type.fields:
            # Only check select fields with transitions defined
            if field_def.type != "select" or field_def.transitions is None:
                continue

            if field_def.name not in data:
                continue

            current_value = current_data.get(field_def.name)
            new_value = data[field_def.name]

            # Skip if value not changing
            if current_value == new_value:
                continue

            # Check if transition is allowed
            if not field_def.is_transition_allowed(current_value or "", new_value or ""):
                raise ValueError(
                    f"Invalid status transition: cannot change {field_def.label or field_def.name} "
                    f"from '{current_value}' to '{new_value}'"
                )

    def serialize(self, entity: Entity) -> dict[str, Any]:
        """Serialize entity to dict."""
        data = {
            "id": entity.id,
            "type": entity.type,
            "created_at": entity.created_at.isoformat() if entity.created_at else None,
            "updated_at": entity.updated_at.isoformat() if entity.updated_at else None,
            "created_by": entity.created_by,
            "updated_by": entity.updated_by,
        }

        # Add field values
        for ev in entity.values:
            data[ev.field_name] = ev.value

        return data



"""Entity API endpoints - unified CRUD for all content types.

This module provides RESTful API endpoints for managing entities (content items)
in the CMS. It supports CRUD operations for any registered content type.

Example content types: post, page, category, tag, etc.
"""

from typing import Any

from fastapi import APIRouter, Depends, HTTPException, Query, Request
from pydantic import BaseModel, Field
from sqlalchemy.ext.asyncio import AsyncSession

from ..database import get_db
from ..models.entity import Entity
from ..rate_limit import limiter
from ..services.auth import AuthService
from ..services.entity import EntityService, QueryParams
from ..services.field import field_service
from .auth import get_session_token

router = APIRouter(prefix="/entities", tags=["Entities"])


async def _get_optional_user(request: Request, db: AsyncSession) -> Entity | None:
    """Get current user if authenticated, None otherwise."""
    token = get_session_token(request)
    if not token:
        return None
    auth_svc = AuthService(db)
    return await auth_svc.get_current_user(token)


class EntityCreate(BaseModel):
    """Request body for creating an entity.

    Example:
        ```json
        {
            "data": {
                "title": "My First Post",
                "slug": "my-first-post",
                "body": [{"type": "paragraph", "data": {"text": "Hello world!"}}],
                "status": "draft"
            }
        }
        ```
    """

    data: dict[str, Any] = Field(
        default_factory=dict,
        description="Entity field values as key-value pairs",
        examples=[{"title": "My First Post", "slug": "my-first-post", "status": "draft"}],
    )


class EntityUpdate(BaseModel):
    """Request body for updating an entity.

    Only include fields that should be updated.
    """

    data: dict[str, Any] = Field(
        default_factory=dict,
        description="Entity field values to update",
        examples=[{"status": "published"}],
    )


class EntityResponse(BaseModel):
    """Response for a single entity.

    Example:
        ```json
        {
            "id": "abc123",
            "type": "post",
            "title": "My First Post",
            "slug": "my-first-post",
            "status": "published",
            "created_at": "2024-01-15T10:30:00Z",
            "updated_at": "2024-01-15T10:30:00Z"
        }
        ```
    """

    id: str = Field(..., description="Unique entity identifier")
    type: str = Field(..., description="Content type name")
    created_at: str = Field(..., description="Creation timestamp (ISO 8601)")
    updated_at: str = Field(..., description="Last update timestamp (ISO 8601)")
    created_by: str | None = Field(None, description="Creator user ID")
    updated_by: str | None = Field(None, description="Last updater user ID")
    data: dict[str, Any] = Field(default_factory=dict, description="Entity field values")

    class Config:
        from_attributes = True


class EntityListResponse(BaseModel):
    """Response for paginated entity list.

    Example:
        ```json
        {
            "items": [{"id": "abc123", "type": "post", "title": "My Post"}],
            "total": 42,
            "page": 1,
            "per_page": 20,
            "pages": 3
        }
        ```
    """

    items: list[dict[str, Any]] = Field(..., description="List of entities")
    total: int = Field(..., description="Total number of entities")
    page: int = Field(..., description="Current page number")
    per_page: int = Field(..., description="Items per page")
    pages: int = Field(..., description="Total number of pages")


@router.get(
    "/{type_name}",
    response_model=EntityListResponse,
    summary="List entities",
    description="Retrieve a paginated list of entities of the specified content type.",
    responses={
        200: {
            "description": "Paginated list of entities",
            "content": {
                "application/json": {
                    "example": {
                        "items": [
                            {
                                "id": "abc123",
                                "type": "post",
                                "title": "Hello World",
                                "status": "published",
                            }
                        ],
                        "total": 42,
                        "page": 1,
                        "per_page": 20,
                        "pages": 3,
                    }
                }
            },
        },
        404: {"description": "Content type not found"},
    },
)
@limiter.limit("100/minute")
async def list_entities(
    request: Request,
    type_name: str,
    page: int = Query(1, ge=1, description="Page number"),
    per_page: int = Query(20, ge=1, le=100, description="Items per page (max 100)"),
    sort: str = Query("-created_at", description="Sort field (prefix with - for descending)"),
    db: AsyncSession = Depends(get_db),
) -> EntityListResponse:
    """List entities of a specific type.

    Retrieves a paginated list of entities. Supports sorting by any field.

    - Use `-field` for descending order (e.g., `-created_at`)
    - Use `field` for ascending order (e.g., `title`)
    """
    # Validate type exists
    ct = field_service.get_content_type(type_name)
    if not ct:
        raise HTTPException(status_code=404, detail=f"Content type '{type_name}' not found")

    service = EntityService(db)
    params = QueryParams(page=page, per_page=per_page, sort=sort)

    entities = await service.find(type_name, params)
    total = await service.count(type_name, params)
    pages = (total + per_page - 1) // per_page

    return EntityListResponse(
        items=[service.serialize(e) for e in entities],
        total=total,
        page=page,
        per_page=per_page,
        pages=pages,
    )


@router.post(
    "/{type_name}",
    summary="Create entity",
    description="Create a new entity of the specified content type.",
    responses={
        200: {
            "description": "Created entity",
            "content": {
                "application/json": {
                    "example": {
                        "id": "abc123",
                        "type": "post",
                        "title": "My New Post",
                        "slug": "my-new-post",
                        "created_at": "2024-01-15T10:30:00Z",
                    }
                }
            },
        },
        400: {"description": "Validation error"},
        404: {"description": "Content type not found"},
    },
)
@limiter.limit("100/minute")
async def create_entity(
    request: Request,
    type_name: str,
    body: EntityCreate,
    db: AsyncSession = Depends(get_db),
) -> dict[str, Any]:
    """Create a new entity.

    Creates a new entity with the provided field values.
    Required fields must be included in the request body.
    """
    # Validate type exists
    ct = field_service.get_content_type(type_name)
    if not ct:
        raise HTTPException(status_code=404, detail=f"Content type '{type_name}' not found")

    # Get current user for audit trail
    user = await _get_optional_user(request, db)
    user_id = user.id if user else None

    service = EntityService(db)

    try:
        entity = await service.create(type_name, body.data, user_id=user_id)
        return service.serialize(entity)
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.get(
    "/{type_name}/{entity_id}",
    summary="Get entity",
    description="Retrieve a single entity by its ID.",
    responses={
        200: {
            "description": "Entity details",
            "content": {
                "application/json": {
                    "example": {
                        "id": "abc123",
                        "type": "post",
                        "title": "Hello World",
                        "slug": "hello-world",
                        "status": "published",
                        "created_at": "2024-01-15T10:30:00Z",
                    }
                }
            },
        },
        404: {"description": "Entity not found"},
    },
)
@limiter.limit("100/minute")
async def get_entity(
    request: Request,
    type_name: str,
    entity_id: str,
    db: AsyncSession = Depends(get_db),
) -> dict[str, Any]:
    """Get a single entity by ID.

    Returns all fields for the specified entity.
    """
    service = EntityService(db)
    entity = await service.get(entity_id)

    if not entity:
        raise HTTPException(status_code=404, detail="Entity not found")

    if entity.type != type_name:
        raise HTTPException(status_code=404, detail="Entity not found")

    return service.serialize(entity)


@router.put(
    "/{type_name}/{entity_id}",
    summary="Update entity",
    description="Update an existing entity with new field values.",
    responses={
        200: {
            "description": "Updated entity",
            "content": {
                "application/json": {
                    "example": {
                        "id": "abc123",
                        "type": "post",
                        "title": "Updated Title",
                        "status": "published",
                        "updated_at": "2024-01-15T12:00:00Z",
                    }
                }
            },
        },
        400: {"description": "Validation error"},
        404: {"description": "Entity not found"},
    },
)
@limiter.limit("100/minute")
async def update_entity(
    request: Request,
    type_name: str,
    entity_id: str,
    body: EntityUpdate,
    db: AsyncSession = Depends(get_db),
) -> dict[str, Any]:
    """Update an entity.

    Only fields included in the request body will be updated.
    Omitted fields retain their current values.
    """
    service = EntityService(db)
    entity = await service.get(entity_id)

    if not entity:
        raise HTTPException(status_code=404, detail="Entity not found")

    if entity.type != type_name:
        raise HTTPException(status_code=404, detail="Entity not found")

    # Get current user for audit trail
    user = await _get_optional_user(request, db)
    user_id = user.id if user else None

    try:
        updated = await service.update(entity_id, body.data, user_id=user_id)
        return service.serialize(updated)
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.delete(
    "/{type_name}/{entity_id}",
    summary="Delete entity",
    description="Delete an entity (soft delete by default, use hard=true for permanent deletion).",
    responses={
        200: {
            "description": "Deletion confirmation",
            "content": {"application/json": {"example": {"status": "deleted", "id": "abc123"}}},
        },
        404: {"description": "Entity not found"},
    },
)
@limiter.limit("100/minute")
async def delete_entity(
    request: Request,
    type_name: str,
    entity_id: str,
    hard: bool = Query(False, description="If true, permanently delete (cannot be undone)"),
    db: AsyncSession = Depends(get_db),
) -> dict[str, str]:
    """Delete an entity.

    By default, performs a soft delete (sets deleted_at timestamp).
    Use `hard=true` to permanently delete the entity.

    **Warning**: Hard delete cannot be undone.
    """
    service = EntityService(db)
    entity = await service.get(entity_id)

    if not entity:
        raise HTTPException(status_code=404, detail="Entity not found")

    if entity.type != type_name:
        raise HTTPException(status_code=404, detail="Entity not found")

    await service.delete(entity_id, hard=hard)
    return {"status": "deleted", "id": entity_id}



"""Admin routes - WordPress import preview and dry-run operations."""

import uuid

from fastapi import APIRouter, Depends, Request, status
from fastapi.responses import JSONResponse
from sqlalchemy.ext.asyncio import AsyncSession

from ..database import get_db
from ..models import Entity
from ..rate_limit import limiter
from ..schemas.import_schema import ErrorResponse
from ..utils import require_feature
from .helpers import require_admin

router = APIRouter()


@router.post(
    "/import/{job_id}/dry-run",
    responses={
        404: {"model": ErrorResponse, "description": "Job not found"},
        500: {"model": ErrorResponse, "description": "Server error"},
    },
)
@limiter.limit("5/minute")
async def dry_run_import(
    request: Request,
    job_id: str,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Run a dry-run simulation of the import without making changes."""
    require_feature("wordpress_import")
    from ..services.wordpress_import import WordPressImportService

    request_id = str(uuid.uuid4())[:8]

    try:
        import_svc = WordPressImportService(db)
        job = await import_svc.get_job(job_id)

        if not job:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content=ErrorResponse(
                    error="Job not found",
                    code="JOB_NOT_FOUND",
                    request_id=request_id,
                ).model_dump(),
            )

        # Run dry-run
        result = await import_svc.dry_run(job_id)

        if not result:
            return JSONResponse(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                content=ErrorResponse(
                    error="Dry-run failed",
                    code="DRY_RUN_FAILED",
                    request_id=request_id,
                ).model_dump(),
            )

        # Convert to JS-expected format
        summary = result.get("summary", {})
        counts = {
            "new": sum(s.get("new", 0) for s in summary.values()),
            "skip": sum(s.get("duplicates", 0) for s in summary.values()),
            "error": len(result.get("errors", [])),
            "update": 0,
        }
        # Build items with status for each type
        items = {}
        for entity_type, stats in summary.items():
            items[entity_type] = [
                {"status": "new"} for _ in range(stats.get("new", 0))
            ] + [
                {"status": "skip"} for _ in range(stats.get("duplicates", 0))
            ]

        return {
            "success": True,
            "job_id": job_id,
            "counts": counts,
            "items": items,
            "warnings": result.get("warnings", []),
            "errors": result.get("errors", []),
            "duplicates": result.get("duplicates", []),
            "has_errors": len(result.get("errors", [])) > 0,
            "request_id": request_id,
        }

    except Exception as e:
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ErrorResponse(
                error=str(e),
                code="INTERNAL_ERROR",
                request_id=request_id,
            ).model_dump(),
        )


@router.post(
    "/import/{job_id}/preview",
    responses={
        404: {"model": ErrorResponse, "description": "Job not found"},
        500: {"model": ErrorResponse, "description": "Server error"},
    },
)
@limiter.limit("5/minute")
async def preview_import(
    request: Request,
    job_id: str,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Preview import by creating a small number of sample items."""
    require_feature("wordpress_import")
    from ..services.wordpress_import import WordPressImportService

    request_id = str(uuid.uuid4())[:8]

    try:
        import_svc = WordPressImportService(db)
        job = await import_svc.get_job(job_id)

        if not job:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content=ErrorResponse(
                    error="Job not found",
                    code="JOB_NOT_FOUND",
                    request_id=request_id,
                ).model_dump(),
            )

        result = await import_svc.preview_import(job_id, limit=3)

        if not result or not result.get("success"):
            return JSONResponse(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                content=ErrorResponse(
                    error=result.get("error", "Preview failed") if result else "Preview failed",
                    code="PREVIEW_FAILED",
                    request_id=request_id,
                ).model_dump(),
            )

        return {
            "success": True,
            "job_id": job_id,
            "preview": result,
            "request_id": request_id,
        }

    except Exception as e:
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ErrorResponse(
                error=str(e),
                code="INTERNAL_ERROR",
                request_id=request_id,
            ).model_dump(),
        )


@router.post(
    "/import/{job_id}/preview/confirm",
    responses={
        404: {"model": ErrorResponse, "description": "Job not found"},
        500: {"model": ErrorResponse, "description": "Server error"},
    },
)
@limiter.limit("5/minute")
async def confirm_preview(
    request: Request,
    job_id: str,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Confirm preview items and finalize them."""
    from ..services.wordpress_import import WordPressImportService

    request_id = str(uuid.uuid4())[:8]

    try:
        import_svc = WordPressImportService(db)
        result = await import_svc.confirm_preview(job_id)

        if not result or not result.get("success"):
            return JSONResponse(
                status_code=status.HTTP_400_BAD_REQUEST,
                content=ErrorResponse(
                    error=result.get("error", "Confirm failed") if result else "Confirm failed",
                    code="CONFIRM_FAILED",
                    request_id=request_id,
                ).model_dump(),
            )

        return {
            "success": True,
            "confirmed": result.get("confirmed", 0),
            "request_id": request_id,
        }

    except Exception as e:
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ErrorResponse(
                error=str(e),
                code="INTERNAL_ERROR",
                request_id=request_id,
            ).model_dump(),
        )


@router.post(
    "/import/{job_id}/preview/discard",
    responses={
        404: {"model": ErrorResponse, "description": "Job not found"},
        500: {"model": ErrorResponse, "description": "Server error"},
    },
)
@limiter.limit("5/minute")
async def discard_preview(
    request: Request,
    job_id: str,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Discard preview items and delete them."""
    from ..services.wordpress_import import WordPressImportService

    request_id = str(uuid.uuid4())[:8]

    try:
        import_svc = WordPressImportService(db)
        result = await import_svc.discard_preview(job_id)

        if not result or not result.get("success"):
            return JSONResponse(
                status_code=status.HTTP_400_BAD_REQUEST,
                content=ErrorResponse(
                    error=result.get("error", "Discard failed") if result else "Discard failed",
                    code="DISCARD_FAILED",
                    request_id=request_id,
                ).model_dump(),
            )

        return {
            "success": True,
            "discarded": result.get("discarded", 0),
            "request_id": request_id,
        }

    except Exception as e:
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ErrorResponse(
                error=str(e),
                code="INTERNAL_ERROR",
                request_id=request_id,
            ).model_dump(),
        )


@router.post(
    "/import/{job_id}/resume",
    responses={
        400: {"model": ErrorResponse, "description": "Cannot resume job"},
        404: {"model": ErrorResponse, "description": "Job not found"},
        500: {"model": ErrorResponse, "description": "Server error"},
    },
)
@limiter.limit("3/minute")
async def resume_import(
    request: Request,
    job_id: str,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Resume a failed or cancelled import job from checkpoint."""
    require_feature("wordpress_import")
    import asyncio

    from ..services.wordpress_import import WordPressImportService

    request_id = str(uuid.uuid4())[:8]

    try:
        import_svc = WordPressImportService(db)
        job = await import_svc.get_job(job_id)

        if not job:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content=ErrorResponse(
                    error="Import job not found",
                    code="JOB_NOT_FOUND",
                    request_id=request_id,
                ).model_dump(),
            )

        # Check if job can be resumed
        from ..models import ImportJobStatus

        if job.status not in (
            ImportJobStatus.FAILED,
            ImportJobStatus.CANCELLED,
        ):
            return JSONResponse(
                status_code=status.HTTP_400_BAD_REQUEST,
                content=ErrorResponse(
                    error=f"Cannot resume job with status: {job.status}. Only failed or cancelled jobs can be resumed.",
                    code="INVALID_STATUS",
                    request_id=request_id,
                ).model_dump(),
            )

        # Get checkpoint info
        checkpoint = job.checkpoint or {}
        last_phase = checkpoint.get("last_phase", "beginning")

        # Start resume in background
        asyncio.create_task(import_svc.resume_import(job_id))

        return {
            "success": True,
            "job_id": job_id,
            "message": f"Import resumed from phase: {last_phase}",
            "checkpoint": {
                "last_phase": last_phase,
                "authors_processed": len(checkpoint.get("authors", [])),
                "categories_processed": len(checkpoint.get("categories", [])),
                "tags_processed": len(checkpoint.get("tags", [])),
                "media_processed": len(checkpoint.get("media", [])),
                "posts_processed": len(checkpoint.get("posts", [])),
                "pages_processed": len(checkpoint.get("pages", [])),
                "menus_processed": len(checkpoint.get("menus", [])),
            },
            "request_id": request_id,
        }

    except Exception as e:
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=ErrorResponse(
                error=str(e),
                code="INTERNAL_ERROR",
                request_id=request_id,
            ).model_dump(),
        )



"""Admin menu routes."""

import structlog

from fastapi import APIRouter, Depends, Form, Request
from fastapi.responses import HTMLResponse, RedirectResponse
from sqlalchemy.ext.asyncio import AsyncSession

from ..database import get_db
from ..models import Entity
from ..services.entity import EntityService
from ..utils import require_feature
from .helpers import templates, require_admin, get_context

logger = structlog.get_logger()

router = APIRouter()


@router.get("/menus", response_class=HTMLResponse)
async def menu_list(
    request: Request,
    location: str = "header",
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Menu management page."""
    require_feature("menu")
    from ..services.menu import MenuService

    menu_svc = MenuService(db)

    # Get menu items for the selected location with error handling
    try:
        items = await menu_svc.get_flat_menu_items(location)
        menu_tree = await menu_svc.get_menu(location)
        has_db_items = await menu_svc.has_db_menu(location)
    except Exception as e:
        logger.error("menu_list_error", location=location, error=str(e))
        items = []
        menu_tree = []
        has_db_items = False

    context = await get_context(request, db, current_user, "menus")
    context.update(
        {
            "items": items,
            "menu_tree": [m.to_dict() for m in menu_tree],
            "current_location": location,
            "locations": [
                {"value": "header", "label": "ヘッダー"},
                {"value": "footer", "label": "フッター"},
                {"value": "sidebar", "label": "サイドバー"},
            ],
            "has_db_items": has_db_items,
            "message": request.query_params.get("message"),
        }
    )

    return templates.TemplateResponse("admin/menus.html", context)


@router.post("/menus/import", response_class=HTMLResponse)
async def menu_import_yaml(
    request: Request,
    location: str = Form(...),
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Import menu items from YAML config to database."""
    require_feature("menu")
    from ..services.menu import MenuService

    menu_svc = MenuService(db)
    entity_svc = EntityService(db)
    user_data = entity_svc.serialize(current_user)

    count = await menu_svc.import_from_yaml(location, user_id=user_data.get("id"))

    return RedirectResponse(
        url=f"/admin/menus?location={location}&message={count}件のメニュー項目をインポートしました",
        status_code=303,
    )


@router.post("/menus/item", response_class=HTMLResponse)
async def menu_item_create(
    request: Request,
    location: str = Form(...),
    label: str = Form(...),
    url: str = Form("#"),
    target: str = Form("_self"),
    link_type: str = Form("custom"),
    linked_entity_id: str = Form(""),
    icon: str = Form(""),
    parent_id: str = Form(""),
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Create a new menu item."""
    require_feature("menu")
    from ..services.menu import MenuService

    menu_svc = MenuService(db)
    entity_svc = EntityService(db)
    user_data = entity_svc.serialize(current_user)

    await menu_svc.create_menu_item(
        location=location,
        label=label,
        url=url,
        target=target,
        icon=icon,
        link_type=link_type,
        linked_entity_id=linked_entity_id if linked_entity_id else None,
        parent_id=parent_id if parent_id else None,
        user_id=user_data.get("id"),
    )

    return RedirectResponse(
        url=f"/admin/menus?location={location}&message=メニュー項目を作成しました",
        status_code=303,
    )


@router.post("/menus/item/{item_id}", response_class=HTMLResponse)
async def menu_item_update(
    request: Request,
    item_id: str,
    location: str = Form(...),
    label: str = Form(...),
    url: str = Form("#"),
    target: str = Form("_self"),
    link_type: str = Form("custom"),
    linked_entity_id: str = Form(""),
    icon: str = Form(""),
    parent_id: str = Form(""),
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Update a menu item."""
    require_feature("menu")
    from ..services.menu import MenuService

    menu_svc = MenuService(db)
    entity_svc = EntityService(db)
    user_data = entity_svc.serialize(current_user)

    await menu_svc.update_menu_item(
        menu_item_id=item_id,
        data={
            "label": label,
            "url": url,
            "target": target,
            "link_type": link_type,
            "linked_entity_id": linked_entity_id,
            "icon": icon,
        },
        parent_id=parent_id if parent_id else None,
        user_id=user_data.get("id"),
    )

    return RedirectResponse(
        url=f"/admin/menus?location={location}&message=メニュー項目を更新しました",
        status_code=303,
    )


@router.delete("/menus/item/{item_id}")
async def menu_item_delete(
    item_id: str,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Delete a menu item."""
    require_feature("menu")
    from ..services.menu import MenuService

    # YAML items cannot be deleted - must import to DB first
    if item_id.startswith("yaml_"):
        return HTMLResponse(
            content="YAML設定のメニューは削除できません。先に「Import from YAML」でDBにインポートしてください。",
            status_code=400,
        )

    menu_svc = MenuService(db)
    entity_svc = EntityService(db)
    user_data = entity_svc.serialize(current_user)

    deleted = await menu_svc.delete_menu_item(item_id, user_id=user_data.get("id"))
    if not deleted:
        return HTMLResponse(
            content="メニュー項目が見つかりません。",
            status_code=404,
        )
    return HTMLResponse(content="", status_code=200)


@router.post("/menus/reorder")
async def menu_reorder(
    request: Request,
    db: AsyncSession = Depends(get_db),
    current_user: Entity = Depends(require_admin),
):
    """Reorder menu items via AJAX."""
    require_feature("menu")
    from ..services.menu import MenuService

    menu_svc = MenuService(db)
    entity_svc = EntityService(db)
    user_data = entity_svc.serialize(current_user)

    body = await request.json()
    location = body.get("location", "header")
    items = body.get("items", [])

    await menu_svc.reorder_menu_items(location, items, user_id=user_data.get("id"))

    return {"status": "ok"}



"""Focomy schemas."""

from .import_schema import (
    AnalyzeRequest,
    ConnectionTestRequest,
    ConnectionTestResponse,
    ErrorResponse,
    ImportAnalysisResponse,
    ImportStatusResponse,
    StartImportResponse,
)

__all__ = [
    "ConnectionTestRequest",
    "ConnectionTestResponse",
    "AnalyzeRequest",
    "ImportAnalysisResponse",
    "StartImportResponse",
    "ImportStatusResponse",
    "ErrorResponse",
]



"""Channel service - Default channel management."""

import structlog

from sqlalchemy.ext.asyncio import AsyncSession

from .entity import EntityService

logger = structlog.get_logger(__name__)

DEFAULT_CHANNEL_SLUG = "posts"


async def get_or_create_posts_channel(db: AsyncSession) -> str:
    """Get or create the default 'posts' channel.

    Returns:
        Channel entity ID
    """
    entity_svc = EntityService(db)

    # Try to find existing posts channel
    entities = await entity_svc.find(
        "channel",
        filters={"slug": DEFAULT_CHANNEL_SLUG},
        limit=1,
    )

    if entities:
        channel_id = entities[0].id
        logger.debug("found_posts_channel", channel_id=channel_id)
        return channel_id

    # Create posts channel
    entity = await entity_svc.create(
        "channel",
        {
            "title": "Posts",
            "slug": DEFAULT_CHANNEL_SLUG,
            "description": "Default channel for posts",
            "sort_order": 0,
        },
    )

    logger.info("created_posts_channel", channel_id=entity.id)
    return entity.id


def is_protected_channel(slug: str) -> bool:
    """Check if a channel is protected from deletion.

    Args:
        slug: Channel slug

    Returns:
        True if protected
    """
    return slug == DEFAULT_CHANNEL_SLUG



"""Search API endpoints."""

from fastapi import APIRouter, Depends, Query
from sqlalchemy.ext.asyncio import AsyncSession

from ..database import get_db
from ..services.search import SearchService, get_search_suggestions

router = APIRouter(prefix="/search", tags=["search"])


@router.get("")
async def search(
    q: str = Query(..., min_length=2, description="Search query"),
    types: str = Query(None, description="Comma-separated content types"),
    page: int = Query(1, ge=1),
    per_page: int = Query(20, ge=1, le=100),
    db: AsyncSession = Depends(get_db),
):
    """Search content across the site.

    Args:
        q: Search query (minimum 2 characters)
        types: Optional comma-separated list of content types to search
        page: Page number (1-based)
        per_page: Results per page

    Returns:
        Search results with pagination info
    """
    search_svc = SearchService(db)

    type_list = None
    if types:
        type_list = [t.strip() for t in types.split(",") if t.strip()]

    offset = (page - 1) * per_page
    results, total = await search_svc.search(
        query=q,
        types=type_list,
        limit=per_page,
        offset=offset,
    )

    return {
        "query": q,
        "results": [
            {
                "id": r.entity_id,
                "type": r.entity_type,
                "title": r.title,
                "excerpt": r.excerpt,
                "score": r.score,
                "url": r.url,
            }
            for r in results
        ],
        "pagination": {
            "page": page,
            "per_page": per_page,
            "total": total,
            "total_pages": (total + per_page - 1) // per_page,
        },
    }


@router.get("/suggestions")
async def search_suggestions(
    q: str = Query(..., min_length=2, description="Search query prefix"),
    limit: int = Query(5, ge=1, le=10),
    db: AsyncSession = Depends(get_db),
):
    """Get search suggestions for autocomplete.

    Args:
        q: Query prefix (minimum 2 characters)
        limit: Maximum suggestions

    Returns:
        List of suggestion strings
    """
    suggestions = await get_search_suggestions(db, q, limit)
    return {"suggestions": suggestions}



"""
Index Service - Create database indexes for indexed fields.

Reads YAML content type definitions and creates appropriate indexes.
"""

from typing import Any

from sqlalchemy import text
from sqlalchemy.ext.asyncio import AsyncSession

from .field import field_service


class IndexService:
    """Service for managing database indexes based on YAML field definitions."""

    def __init__(self, db: AsyncSession):
        self.db = db

    async def create_indexes_for_all_types(self) -> dict[str, list[str]]:
        """
        Create indexes for all indexed fields in all content types.

        Returns:
            Dict mapping content type to list of created index names
        """
        results = {}
        content_types = field_service.get_all_content_types()

        for type_name, ct in content_types.items():
            created = await self.create_indexes_for_type(type_name, ct)
            if created:
                results[type_name] = created

        return results

    async def create_indexes_for_type(self, type_name: str, content_type) -> list[str]:
        """
        Create indexes for indexed fields in a content type.

        Returns:
            List of created index names
        """
        created = []

        for field in content_type.fields:
            if not field.indexed:
                continue

            index_name = await self._create_field_index(type_name, field)
            if index_name:
                created.append(index_name)

        return created

    async def _create_field_index(self, type_name: str, field) -> str | None:
        """
        Create an index for a single field.

        Creates a partial index on entity_values for the specific field_name
        and value column based on field type.
        """
        # Determine which value column to index based on field type
        storage_type = self._get_storage_type(field.type)
        value_column = f"value_{storage_type}"

        # Create a unique index name
        index_name = f"idx_{type_name}_{field.name}"

        # Check if index already exists
        check_sql = text(
            """
            SELECT 1 FROM pg_indexes
            WHERE indexname = :index_name
        """
        )
        result = await self.db.execute(check_sql, {"index_name": index_name})
        if result.scalar():
            return None  # Index already exists

        # Create partial index for this field
        # This indexes only rows where field_name matches, making it efficient
        create_sql = text(
            f"""
            CREATE INDEX IF NOT EXISTS {index_name}
            ON entity_values ({value_column})
            WHERE field_name = :field_name
        """
        )

        try:
            await self.db.execute(create_sql, {"field_name": field.name})
            await self.db.commit()
            return index_name
        except Exception as e:
            await self.db.rollback()
            print(f"Warning: Could not create index {index_name}: {e}")
            return None

    def _get_storage_type(self, field_type: str) -> str:
        """Map field type to storage column type."""
        type_mapping = {
            "string": "text",
            "text": "text",
            "slug": "text",
            "email": "text",
            "url": "text",
            "password": "text",
            "integer": "int",
            "float": "float",
            "boolean": "int",
            "datetime": "datetime",
            "date": "datetime",
            "select": "text",
            "multiselect": "json",
            "blocks": "json",
            "media": "text",
            "json": "json",
        }
        return type_mapping.get(field_type, "text")

    async def list_custom_indexes(self) -> list[dict[str, Any]]:
        """List all custom indexes created by this service."""
        sql = text(
            """
            SELECT indexname, tablename, indexdef
            FROM pg_indexes
            WHERE schemaname = 'public'
            AND indexname LIKE 'idx_%_%'
            AND tablename = 'entity_values'
            ORDER BY indexname
        """
        )
        result = await self.db.execute(sql)
        rows = result.fetchall()

        return [
            {
                "name": row[0],
                "table": row[1],
                "definition": row[2],
            }
            for row in rows
        ]

    async def drop_index(self, index_name: str) -> bool:
        """Drop a custom index."""
        if not index_name.startswith("idx_"):
            return False

        try:
            sql = text(f"DROP INDEX IF EXISTS {index_name}")
            await self.db.execute(sql)
            await self.db.commit()
            return True
        except Exception:
            await self.db.rollback()
            return False
